# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.pivot import Pivot



class Pivot_max(Pivot, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.pivot.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb.addItems_(items, '')

		ctx = self.sb.pivot.tb000.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add('QCheckBox', setText='Reset Pivot Position', setObjectName='chk000', setChecked=True, setToolTip='')
			ctx.add('QCheckBox', setText='Reset Pivot Orientation', setObjectName='chk001', setChecked=True, setToolTip='')
			ctx.add('QCheckBox', setText='Reset XForm', setObjectName='chk013', setToolTip='')

		ctx = self.sb.pivot.tb001.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add('QRadioButton', setText='Component', setObjectName='chk002', setToolTip='Center the pivot on the center of the selected component\'s bounding box')
			ctx.add('QRadioButton', setText='Object', setObjectName='chk003', setChecked=True, setToolTip='Center the pivot on the center of the object\'s bounding box')
			ctx.add('QRadioButton', setText='World', setObjectName='chk004', setToolTip='Center the pivot on world origin.')
			ctx.add('QRadioButton', setText='Object Top', setObjectName='chk005', setToolTip='Move the pivot to the top of the object.')
			ctx.add('QRadioButton', setText='Object Bottom', setObjectName='chk006', setToolTip='Move the pivot to the bottom of the object.')
			ctx.add('QRadioButton', setText='Object Center Left', setObjectName='chk007', setToolTip='Move the pivot to the center left of the object.')
			ctx.add('QRadioButton', setText='Object Center Right', setObjectName='chk008', setToolTip='Move the pivot to the center right of the object.')
			ctx.add('QRadioButton', setText='Object Bottom Left', setObjectName='chk009', setToolTip='Move the pivot to the bottom left of the object.')
			ctx.add('QRadioButton', setText='Object Bottom Right', setObjectName='chk010', setToolTip='Move the pivot to the bottom right of the object.')
			ctx.add('QRadioButton', setText='Object Top Left', setObjectName='chk011', setToolTip='Move the pivot to the top left of the object.')
			ctx.add('QRadioButton', setText='Object Top Right', setObjectName='chk012', setToolTip='Move the pivot to the top right of the object.')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.pivot.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Reset Pivot
		'''
		tb = self.sb.pivot.tb000

		if tb.ctxMenu.chk000: #Reset Pivot Scale
			rt.ResetScale(rt.selection) #Same as Hierarchy/Pivot/Reset Scale.
		
		if tb.ctxMenu.chk001: #Reset Pivot Transform
			rt.ResetTransform(rt.selection) #Same as Hierarchy/Pivot/Reset Transform.

		if tb.ctxMenu.chk013: #reset XForm
			rt.ResetXForm(rt.selection) #Same as the Reset XForm utility in the Utilities tab - applies XForm modifier to node, stores the current transformations in the gizmo and resets the object transformations.
			self.messageBox('ResetXForm '+str([obj.name for obj in rt.selection]))
			return

		# rt.ResetPivot(rt.selection) #Same as Hierarchy/Pivot/Reset Pivot.


	def tb001(self, state=None):
		'''Center Pivot
		'''
		tb = self.sb.pivot.tb001

		component = tb.ctxMenu.chk002.isChecked()
		object_ = tb.ctxMenu.chk003.isChecked()
		world = tb.ctxMenu.chk004.isChecked()
		objectTop = tb.ctxMenu.chk005.isChecked()
		objectBottom = tb.ctxMenu.chk006.isChecked()
		objectCenterLeft = tb.ctxMenu.chk007.isChecked()
		objectCenterRight = tb.ctxMenu.chk008.isChecked()
		objectBottomLeft = tb.ctxMenu.chk009.isChecked()
		objectBottomRight = tb.ctxMenu.chk010.isChecked()
		objectTopLeft = tb.ctxMenu.chk011.isChecked()
		objectTopRight = tb.ctxMenu.chk012.isChecked()

		if component: #Set pivot points to the center of the component's bounding box.
			rt.CenterPivot(rt.selection) #Same as Hierarchy/Pivot/Affect Pivot Only - Center to Object.
		elif object_: ##Set pivot points to the center of the object's bounding box
			self.centerPivot(rt.selection)
		elif world:
			rt.selection.pivot = [0,0,0] #center pivot world 0,0,0
		elif objectTop:
			[setattr(obj, 'pivot', [obj.position.x, obj.position.y, obj.max.z]) for obj in rt.selection] #Move the pivot to the top of the object
		elif objectBottom:
			[setattr(obj, 'pivot', [obj.position.x, obj.position.y, obj.min.z]) for obj in rt.selection] #Move the pivot to the bottom of the object
		elif objectCenterLeft:
			[setattr(obj, 'pivot', [obj.min.x, obj.position.y, obj.position.z]) for obj in rt.selection] #Move the pivot to the Center left of the object
		elif objectCenterRight:
			[setattr(obj, 'pivot', [obj.max.x, obj.position.y, obj.position.z]) for obj in rt.selection] #Move the pivot to the Center right of the object
		elif objectBottomLeft:
			[setattr(obj, 'pivot', [obj.min.x, obj.position.y, obj.min.z]) for obj in rt.selection] #Move the pivot to the Bottom left of the object
		elif objectBottomRight:
			[setattr(obj, 'pivot', [obj.max.x, obj.position.y, obj.min.z]) for obj in rt.selection] #Move the pivot to the Bottom right of the object
		elif objectTopLeft:
			[setattr(obj, 'pivot', [obj.min.x, obj.position.y, obj.max.z]) for obj in rt.selection] #Move the pivot to the Top left of the object
		elif objectTopRight:
			[setattr(obj, 'pivot', [obj.max.x, obj.position.y, obj.max.z]) for obj in rt.selection] #Move the pivot to the Topright of the object


	def b004(self):
		'''Bake Pivot
		'''
		print ('Command does not exist:', __name__)


	def centerPivot(self, objects):
		'''Center the rotation pivot on the given objects.
		'''
		for obj in objects:
			rt.toolMode.coordsys(obj) #Center Pivot Object
			obj.pivot = obj.center





	





#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

# maxEval('max tti')

# maxEval('macros.run \"PolyTools\" \"TransformTools\")




# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.main import Main



class Main_maya(Main, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)












#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# deprecated: -----------------------------------

		# 	node history
		# 	selection = pm.ls(sl=1, objectsOnly=1, flatten=1)
		# 	if selection:
		# 		history = selection[0].history()[1:]
		# 		for node in history:
		# 			parent = tree.add('QLabel', 'History', childHeader=node.name(), refresh=1, setText=node.name())
		# 			# print(parent, node.name())
		# 			attributes = mtk.Node.getAttributesMEL(node) #get dict containing attributes:values of the history node.
		# 			spinboxes = [tree.add('QDoubleSpinBox', parent, refresh=1, setSpinBoxByValue_=[k, v])
		# 				for k, v in attributes.items() 
		# 					if isinstance(v, (float, int, bool))]

		# 			set signal/slot connections:
		# 			wgts.= [tree.add(self.MultiWidget, parent, refresh=1, set_by_value=[k, v])
		# 				for k, v in attributes.items() 
		# 					if isinstance(v, (float, int, bool))]

		# 			for multiWidget in wgts.
		# 				attr = multiWidget.children_(index=0).text()
		# 				w = multiWidget.children_(index=1)
		# 				type_ = w.__class__.__name__

		# 				if type_ in ['QSpinBox', 'QDoubleSpinBox']:
		# 					w.valueChanged.connect(
		# 						lambda value, widget=w, node=node: mtk.Node.setAttributesMEL(node, {attr:w.value()}))

		# 			[w.valueChanged.connect(
		# 				lambda value, widget=w, node=node: mtk.Node.setAttributesMEL(node, {widget.prefix().rstrip(': '):value})) 
		# 					for w in spinboxes] #set signal/slot connections


# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.cameras import Cameras


class Cameras_maya(Cameras, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		# tree = self.sb.cameras_lower_submenu.tree000
		# l = ['Camera Sequencer', 'Camera Set Editor']
		# [tree.add('QLabel', 'Editors', setText=s) for s in l]

		# l = ['Exclusive to Camera', 'Hidden from Camera', 'Remove from Exclusive', 'Remove from Hidden', 'Remove All for Camera', 'Remove All']
		# [tree.add('QLabel', 'Per Camera Visibility', setText=s) for s in l]


	@property
	def clippingMenu(self):
		'''Menu: Camera clip plane settings.

		:Return:
			(obj) menu as a property.
		'''
		if not hasattr(self, '_clippingMenu'):
			self._clippingMenu = self.sb.Menu(self.sb.cameras, position='cursorPos')
			self._clippingMenu.add('QPushButton', setText='Auto Clip', setObjectName='chk000', setCheckable=True, setToolTip='When Auto Clip is ON, geometry closer to the camera than 3 units is not displayed. Turn OFF to manually define.')
			self._clippingMenu.add('QDoubleSpinBox', setPrefix='Far Clip:  ', setObjectName='s000', setMinMax_='.01-10 step.1', setToolTip='Adjust the current cameras near clipping plane.')
			self._clippingMenu.add('QSpinBox', setPrefix='Near Clip: ', setObjectName='s001', setMinMax_='10-10000 step1', setToolTip='Adjust the current cameras far clipping plane.')

		#set widget states for the current activeCamera
		activeCamera = self.getCurrentCam()
		if not activeCamera:
			self.sb.toggleWidgets(self._clippingMenu, setDisabled='s000-1,chk000')

		elif pm.viewClipPlane(activeCamera, query=1, autoClipPlane=1): #if autoClipPlane is active:
			self._clippingMenu.chk000.setChecked(True)
			self.sb.toggleWidgets(self._clippingMenu, setDisabled='s000-1')

		nearClip = pm.viewClipPlane(activeCamera, query=1, nearClipPlane=1) if activeCamera else 1.0
		farClip = pm.viewClipPlane(activeCamera, query=1, farClipPlane=1) if activeCamera else 1000.0

		self._clippingMenu.s000.setValue(nearClip)
		self._clippingMenu.s001.setValue(farClip)

		return self._clippingMenu


	def chk000(self, state=None):
		'''Camera Clipping: Auto Clip
		'''
		if self.clippingMenu.chk000.isChecked():
			self.sb.toggleWidgets(self.clippingMenu, setDisabled='s000-1')
		else:
			self.sb.toggleWidgets(self.clippingMenu, setEnabled='s000-1')

		activeCamera = self.getCurrentCam()
		if not activeCamera:
			self.messageBox('No Active Camera.')
			return

		pm.viewClipPlane(activeCamera, autoClipPlane=True)


	def s000(self, value=None):
		'''Camera Clipping: Near Clip
		'''
		value = self.clippingMenu.s000.value()

		activeCamera = self.getCurrentCam()
		if not activeCamera:
			self.messageBox('No Active Camera.')
			return

		pm.viewClipPlane(activeCamera, nearClipPlane=widget.value())


	def s001(self, value=None):
		'''Camera Clipping: Far Clip
		'''
		value = self.clippingMenu.s001.value()

		activeCamera = self.getCurrentCam()
		if not activeCamera:
			self.messageBox('No Active Camera.')
			return

		pm.viewClipPlane(activeCamera, farClipPlane=widget.value())


	def tree000(self, wItem=None, column=None):
		'''
		'''
		tree = self.sb.cameras_lower_submenu.tree000

		if not any([wItem, column]): # code here will run before each show event. generally used to refresh tree contents. -----------------------------
			try:
				cameras = pm.ls(type=('camera'), l=True) #Get all cameras
				startup_cameras = [camera for camera in cameras if pm.camera(camera.parent(0), startupCamera=True, q=True)] #filter all startup / default cameras
				non_startup_cameras_pynodes = list(set(cameras) - set(startup_cameras)) #get non-default cameras. these are all PyNodes
				non_startup_cameras_transform_pynodes = map(lambda x: x.parent(0), non_startup_cameras_pynodes) #get respective transform names
				non_startup_cameras = map(str, non_startup_cameras_pynodes) #non-PyNode, regular string name list
				non_startup_cameras_transforms = map(str, non_startup_cameras_transform_pynodes)

			except AttributeError:
				non_startup_cameras=[]
			[tree.add('QLabel', 'Cameras', refresh=True, setText=s) for s in non_startup_cameras]
			return

		widget = tree.getWidget(wItem, column)
		text = tree.getWidgetText(wItem, column)
		header = tree.getHeaderFromColumn(column)

		if header=='Create':
			if text=='Custom Camera':
				pm.mel.eval('camera -centerOfInterest 5 -focalLength 35 -lensSqueezeRatio 1 -cameraScale 1 -horizontalFilmAperture 1.41732 -horizontalFilmOffset 0 -verticalFilmAperture 0.94488 -verticalFilmOffset 0 -filmFit Fill -overscan 1 -motionBlur 0 -shutterAngle 144 -nearClipPlane 0.1 -farClipPlane 10000 -orthographic 0 -orthographicWidth 30 -panZoomEnabled 0 -horizontalPan 0 -verticalPan 0 -zoom 1; objectMoveCommand; cameraMakeNode 1 "";')
			if text=='Set Custom Camera':
				pm.mel.eval('string $homeName = `cameraView -camera persp`;') #cameraView -edit -camera persp -setCamera $homeName;
			if text=='Camera From View':
				self.createCameraFromView()

		if header=='Cameras':
			pm.select(text)
			pm.lookThru(text)

		if header=='Editors':
			if text=='Camera Sequencer':
				pm.mel.eval('SequenceEditor;')
			if text=='Camera Set Editor':
				pm.mel.eval('cameraSetEditor;')

		if header=='Per Camera Visibility':
			if text=='Exclusive to Camera':
				pm.mel.eval('SetExclusiveToCamera;') #doPerCameraVisibility 0; Make selected objects exclusive to the selected (or current) camera.
			if text=='Hidden from Camera':
				pm.mel.eval('SetHiddenFromCamera;') #doPerCameraVisibility 1; Make selected objects hidden from the selected (or current) camera.
			if text=='Remove from Exclusive':
				pm.mel.eval('CameraRemoveFromExclusive;') #doPerCameraVisibility 2; Remove selected objects from the selected (or current) camera's exclusive list.
			if text=='Remove from Hidden':
				pm.mel.eval('CameraRemoveFromHidden;') #doPerCameraVisibility 3; Remove the selected objects from the selected (or current) camera's hidden list.
			if text=='Remove All for Camera': #Remove all hidden or exclusive objects for the selected (or current) camera.
				pm.mel.eval('CameraRemoveAll;') #doPerCameraVisibility 4; 
			if text=='Remove All':
				pm.mel.eval('CameraRemoveAllForAll;') #doPerCameraVisibility 5; Remove all hidden or exclusive objects for all cameras.

		if header=='Options':
			if text=='Group Cameras':
				self.groupCameras()
			if text=='Adjust Clipping':
				self.clippingMenu.show()
			if text=='Toggle Safe Frames': #Viewport Safeframes Toggle
				self.toggleSafeFrames()


	def v000(self):
		'''Cameras: Back View
		'''
		try: #if pm.objExists('back'):
			pm.lookThru('back')

		except Exception as error:
			cam, camShape = pm.camera() #create camera
			pm.lookThru(cam)

			pm.rename(cam, 'back')
			pm.viewSet(back=1) #initialize the camera view
			pm.hide(cam)

			grp = pm.ls('cameras', transforms=1)
			if grp and self.isGroup(grp[0]): #add the new cam to 'cameras' group (if it exists).
				pm.parent(cam, 'cameras')


	def v001(self):
		'''Cameras: Top View
		'''
		try:
			pm.lookThru('topShape')

		except Exception as error:
			pm.lookThru('|top')


	def v002(self):
		'''Cameras: Right View
		'''
		try:
			pm.lookThru('sideShape')

		except Exception as error:
			pm.lookThru('|side')


	def v003(self):
		'''Cameras: Left View
		'''
		try: #if pm.objExists('back'):
			pm.lookThru('left')

		except Exception as error:
			cam, camShape = pm.camera() #create camera
			pm.lookThru(cam)

			pm.rename(cam, 'left')
			pm.viewSet(leftSide=1) #initialize the camera view
			pm.hide(cam)

			grp = pm.ls('cameras', transforms=1)
			if grp and self.isGroup(grp[0]): #add the new cam to 'cameras' group (if it exists).
				pm.parent(cam, 'cameras')


	def v004(self):
		'''Cameras: Perspective View
		'''
		try:
			pm.lookThru('perspShape')

		except Exception as error:
			pm.lookThru('|persp')


	def v005(self):
		'''Cameras: Front View
		'''
		try:
			pm.lookThru('frontShape')

		except Exception as error:
			pm.lookThru('|front')


	def v006(self):
		'''Cameras: Bottom View
		'''
		try: #if pm.objExists('back'):
			pm.lookThru('bottom')

		except Exception as error:
			cam, camShape = pm.camera() #create camera
			pm.lookThru(cam)

			pm.rename(cam, 'bottom')
			pm.viewSet(bottom=1) #initialize the camera view
			pm.hide(cam)

			grp = pm.ls('cameras', transforms=1)
			if grp and self.isGroup(grp[0]): #add the new cam to 'cameras' group (if it exists).
				pm.parent(cam, 'cameras')


	def v007(self):
		'''Cameras: Align View
		'''
		selection = pm.ls(sl=1)
		if not selection:
			self.messageBox('Nothing Selected.')
			return

		if not pm.objExists('alignToPoly'): #if no camera exists; create camera
			cam, camShape = pm.camera()
			pm.rename(cam, 'alignToPoly')
			pm.hide(cam)

			grp = pm.ls('cameras', transforms=1)
			if grp and self.isGroup(grp[0]): #add the new cam to 'cameras' group (if it exists).
				pm.parent(cam, 'cameras')

		ortho = int(pm.camera('alignToPoly', query=1, orthographic=1)) #check if camera view is orthoraphic
		if not ortho:
			pm.viewPlace('alignToPoly', ortho=1)

		pm.lookThru('alignToPoly')
		pm.AlignCameraToPolygon()
		pm.viewFit(fitFactor=5.0)


	def groupCameras(self):
		'''Group Default Cameras
		'''
		grp = pm.ls('cameras', transforms=1)
		if grp and self.isGroup(grp[0]): #add the new cam to 'cameras' group (if it exists).
			self.messageBox('Group \'cameras\' already exists.')
			return

		pm.group(world=1, name='cameras')
		pm.hide('cameras')

		cameras = ('side', 'front', 'top', 'persp', 'back', 'bottom', 'left', 'alignToPoly')

		for c in cameras:
			try:
				pm.parent(c, 'cameras')
			except Exception as error:
				pass


	def toggleSafeFrames(self):
		'''Toggle display of the film gate for the current camera.
		'''
		camera = self.getCurrentCam()

		state = pm.camera(camera, query=1, displayResolution=1)
		if state:
			pm.camera(camera, edit=1, displayFilmGate=False, displayResolution=False, overscan=1.0)
		else:
			pm.camera(camera, edit=1, displayFilmGate=False, displayResolution=True, overscan=1.3)


	def getCurrentCam(self):
		'''Get the currently active camera.
		'''
		from maya.OpenMaya import MDagPath
		from maya.OpenMayaUI import M3dView

		view = M3dView.active3dView()
		cam = MDagPath()
		view.getCamera(cam)
		camPath = cam.fullPathName()
		return camPath


	def createCameraFromView(self):
		'''Create a new camera base on the current view.
		'''
		from maya.cmds import getPanel #pymel getPanel is broken in ver: 2022.

		curPanel = getPanel(withFocus=True)
		if getPanel(typeOf=curPanel) == "modelPanel":
			camera = pm.modelPanel(curPanel, q=1, cam=1)
			newCamera = pm.duplicate(camera)[0]
			pm.showHidden(newCamera)
			# pm.mel.lookThroughModelPanel(newCamera, curPanel)
			newCamera_ = pm.rename(newCamera, 'Camera')
			print (newCamera_)
			return newCamera_









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------




#deprecated -------------------------------------


# def tree000(self, wItem=None, column=None):
# 		'''

# 		'''
# 		tree = self.sb.cameras.tree000

# 		if not any([wItem, column]):
# 			if not tree.refresh: #static list items -----------
# 				tree.expandOnHover = True
# 				tree.convert(tree.getTopLevelItems(), 'QLabel') #construct the tree using the existing contents.

# 				l = []
# 				[tree.add('QLabel', 'Editors', setText=s) for s in l]
	

# 			#refreshed list items -----------------------------
# 			try:
# 				cameras = pm.ls(type=('camera'), l=True) #Get all cameras
# 				startup_cameras = [camera for camera in cameras if pm.camera(camera.parent(0), startupCamera=True, q=True)] #filter all startup / default cameras
# 				non_startup_cameras_pynodes = list(set(cameras) - set(startup_cameras)) #get non-default cameras. these are all PyNodes
# 				non_startup_cameras_transform_pynodes = map(lambda x: x.parent(0), non_startup_cameras_pynodes) #get respective transform names
# 				non_startup_cameras = map(str, non_startup_cameras_pynodes) #non-PyNode, regular string name list
# 				non_startup_cameras_transforms = map(str, non_startup_cameras_transform_pynodes)
# 			except AttributeError: non_startup_cameras=[]
# 			[tree.add('QLabel', 'Cameras', refresh=True, setText=s) for s in non_startup_cameras]

# 			l = ['Camera Sequencer', 'Camera Set Editor']
# 			[tree.add('QLabel', 'Editors', setText=s) for s in l]
# 			return

# 		widget = tree.getWidget(wItem, column)
# 		text = tree.getWidgetText(wItem, column)
# 		header = tree.getHeaderFromColumn(column)
# 		print(header, text, column)

# 		if header=='Create':
# 			if text=='Custom Camera':
# 				pm.mel.eval('camera -centerOfInterest 5 -focalLength 35 -lensSqueezeRatio 1 -cameraScale 1 -horizontalFilmAperture 1.41732 -horizontalFilmOffset 0 -verticalFilmAperture 0.94488 -verticalFilmOffset 0 -filmFit Fill -overscan 1 -motionBlur 0 -shutterAngle 144 -nearClipPlane 0.1 -farClipPlane 10000 -orthographic 0 -orthographicWidth 30 -panZoomEnabled 0 -horizontalPan 0 -verticalPan 0 -zoom 1; objectMoveCommand; cameraMakeNode 1 "";')
# 			if text=='Set Custom Camera':
# 				pm.mel.eval('string $homeName = `cameraView -camera persp`;') #cameraView -edit -camera persp -setCamera $homeName;
# 			if text=='Camera From View':
# 				print('No Maya Version')

# 		if header=='Cameras':
# 			pm.select(text)
# 			pm.lookThru(text)

# 		if header=='Editors':
# 			if text=='Camera Sequencer':
# 				pm.mel.eval('SequenceEditor;')
# 			if text=='Camera Set Editor':
# 				pm.mel.eval('cameraSetEditor;')

# 		if header=='Options':
# 			if text=='Group Cameras':
# 				self.groupCameras()
# 			if text=='Adjust Clipping':
# 				self.clippingMenu.show()
# 			if text=='Toggle Safe Frames': #Viewport Safeframes Toggle
# 				self.toggleSafeFrames()



	# def cmb000(self, index=-1):
	# 	'''
	# 	Camera Editors

	# 	'''
	# 	cmb = self.sb.cameras.draggable_header.ctxMenu.cmb000
		
	# 	items = ['Camera Sequencer', 'Camera Set Editor']
	# 	contents = cmb.addItems_(items, '')

	# 	if not index:
	# 		index = cmb.currentIndex()
	# 	if index!=0:
	# 		if index==1:
	# 			pm.mel.eval('SequenceEditor;')
	# 		if index==2:
	# 			pm.mel.eval('cameraSetEditor;')
	# 		cmb.setCurrentIndex(0)


	# def cmb001(self, index=-1):
	# 	'''
	# 	Additional Cameras

	# 	'''
	# 	# Get all cameras first
	# 	cameras = pm.ls(type=('camera'), l=True)
	# 	# Let's filter all startup / default cameras
	# 	startup_cameras = [camera for camera in cameras if pm.camera(camera.parent(0), startupCamera=True, q=True)]
	# 	# non-default cameras are easy to find now. Please note that these are all PyNodes
	# 	non_startup_cameras_pynodes = list(set(cameras) - set(startup_cameras))
	# 	# Let's get their respective transform names, just in-case
	# 	non_startup_cameras_transform_pynodes = map(lambda x: x.parent(0), non_startup_cameras_pynodes)
	# 	# Now we can have a non-PyNode, regular string names list of them
	# 	non_startup_cameras = map(str, non_startup_cameras_pynodes)
	# 	non_startup_cameras_transforms = map(str, non_startup_cameras_transform_pynodes)

	# 	cmb = self.sb.cameras.cmb001
		
	# 	contents = cmb.addItems_(non_startup_cameras, "Cameras")

	# 	if not index:
	# 		index = cmb.currentIndex()
	# 	if index!=0:
	# 		pm.select (contents[index])
	# 		cmb.setCurrentIndex(0)


	# def cmb002(self, index=-1):
	# 	'''
	# 	Create

	# 	'''
	# 	cmb = self.sb.cameras.cmb002
		
	# 	items = ['Custom Camera', 'Set Custom Camera', 'Camera From View']
	# 	contents = cmb.addItems_(items, "Create")

	# 	if not index:
	# 		index = cmb.currentIndex()
	# 	if index!=0:
	# 		if index==1:
	# 			pm.mel.eval('cameraView -edit -camera persp -setCamera $homeName;')
	# 		if index==2:
	# 			pm.mel.eval('string $homeName = `cameraView -camera persp`;')
	# 		if index==3:
	# 			pm.mel.eval('print "--no code--")
	# 		cmb.setCurrentIndex(0)


	# def cmb003(self, index=-1):
	# 	'''
	# 	Options

	# 	'''
	# 	cmb = self.sb.cameras.cmb003
		
	# 	items = ['Group Cameras']
	# 	contents = cmb.addItems_(items, "Options")

	# 	if not index:
	# 		index = cmb.currentIndex()
	# 	if index!=0:
	# 		if index==1:
	# 			pm.mel.eval('''
	# 			if (`objExists cameras`)
	# 			{
	# 			  print "Group 'cameras' already exists";
	# 			}
	# 			else
	# 			{
	# 			  group -world -name cameras side front top persp;
	# 			  hide cameras;
	# 			  // Now add non-default cameras to group
	# 			  if (`objExists back`)
	# 			  {
	# 			  	parent back cameras;
	# 			  }
	# 			  if (`objExists bottom`)
	# 			  {
	# 			  	parent bottom cameras;
	# 			  }
	# 			  if (`objExists left`)
	# 			  {
	# 			  	parent left cameras;
	# 			  }
	# 			  if (`objExists alignToPoly`)
	# 			  {
	# 			  	parent alignToPoly cameras;
	# 			  }
	# 			}
	# 			''')
	# 		cmb.setCurrentIndex(0)



# !/usr/bin/python
# coding=utf-8
from PySide2 import QtWidgets, QtCore

from attributes import Attributes
from text import RichText
from menu import MenuInstance



class PushButton_optionBox(QtWidgets.QPushButton, Attributes, RichText):
	'''
	'''
	def __init__(self, parent, showMenuOnMouseOver=False, **kwargs):
		super().__init__(parent)

		#override built-ins
		self.text = self.richText
		self.setText = self.setRichText
		self.sizeHint = self.richTextSizeHint

		self.setStyleSheet(parent.styleSheet())

		# font = self.font()
		# font.setPointSize(font.pointSize()+3)
		# self.setFont(font)

		self.setText('⧉') #default option box text.
		self.setObjectName('{}_optionBox'.format(parent.objectName()))

		self.setMaximumSize(parent.size().height(), parent.size().height())
		parent.setMinimumSize(parent.size().width()-parent.size().height(), parent.size().height())

		self.setAttribute(QtCore.Qt.WA_SetStyle) #Indicates that the widget has a style of its own.
		self.setAttributes(**kwargs)


	def create(self):
		'''
		'''
		self.orig_parent = self.parent() #the parent will change after adding a container and a layout, but we will need the original parent widget later.
		g_parent = self.parent().parent()
		container = QtWidgets.QWidget(g_parent)
		container.setMaximumHeight(self.parent().size().height())
		try:
			removed_wItem = g_parent.layout().replaceWidget(self.parent(), container)
			g_parent.layout().update()
		except AttributeError as error:
			pass

		layout = QtWidgets.QHBoxLayout(container)
		layout.setContentsMargins(0,0,0,0)
		layout.setSpacing(0)
		layout.addWidget(self.orig_parent, 0)
		layout.addWidget(self, 1)

		# self.setStyleSheet('''
		# 	QPushButton {border: 0px solid transparent;}
		# 	''')
		# self.orig_parent.setStyleSheet('''
		# 	QPushButton {border: 0px solid transparent;}
		# 	''')
		# container.setStyleSheet('''
		# 	QWidget {border: 1px solid rgb(50,50,50);}
		# 	''')


	# def enterEvent(self, event):
	# 	'''
	# 	:Parameters:
	# 		event = <QEvent>
	# 	'''
	# 	# if self.showMenuOnMouseOver:
	# 	# 	self.menu_.show()

	# 	return QtWidgets.QPushButton.enterEvent(self, event)


	def mousePressEvent(self, event):
		'''
		:Parameters:
			event = <QEvent>
		'''
		if event.button()==QtCore.Qt.LeftButton:
			self.orig_parent.ctxMenu.show()

		QtWidgets.QPushButton.mousePressEvent(self, event)


	# def leaveEvent(self, event):
	# 	'''
	# 	:Parameters:
	# 		event = <QEvent>
	# 	'''
	# 	if self.showMenuOnMouseOver:
	# 		self.menu_.hide()

	# 	QtWidgets.QPushButton.leaveEvent(self, event)


	# def showEvent(self, event):
	# 	'''
	# 	:Parameters:
	# 		event = <QEvent>
	# 	'''
	# 	if self.menu_.containsMenuItems:
	# 		self.menu_.setTitle(self.text())
	# 		self.menu_.applyButton.show()

	# 	QtWidgets.QPushButton.showEvent(self, event)









if __name__ == "__main__":
	import sys
	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.

	window = QtWidgets.QWidget()
	parent = QtWidgets.QPushButton(window)
	parent.setText('Parent')
	parent.resize(120, 20)

	w = PushButton_optionBox(parent,
		setText='<hl style="color:black;">⧉</hl>'
	)

	window.show()
	sys.exit(app.exec_())



# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

'''
Promoting a widget in designer to use a custom class:
>	In Qt Designer, select all the widgets you want to replace, 
		then right-click them and select 'Promote to...'. 

>	In the dialog:
		Base Class:		Class from which you inherit. ie. QWidget
		Promoted Class:	Name of the class. ie. "MyWidget"
		Header File:	Path of the file (changing the extension .py to .h)  ie. myfolder.mymodule.mywidget.h

>	Then click "Add", "Promote", 
		and you will see the class change from "QWidget" to "MyWidget" in the Object Inspector pane.
'''

# Deprecated: --------------------




# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots
from pythontk import File as File_


class File(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		#set the text for the open last file button to the last file's name.
		list000 = self.sb.file_submenu.list000
		recentFiles = [File_.formatPath(f, 'name') for f in self.getRecentFiles()[:6] if f]
		# list000.setVisible(bool(recentFiles))
		recentFiles = ['fileA', 'fileB', 'fileC']
		list000.addItems(recentFiles)
		# self.sb.file_submenu.b001.setText(File_.formatPath(mostRecentFile, 'name')) if mostRecentFile else self.sb.file_submenu.b001.setVisible(False)

		dh = self.sb.file.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='')
		dh.ctxMenu.add(self.sb.PushButton, setObjectName='tb000', setText='Save', setToolTip='Save the current file.')
		dh.ctxMenu.add(self.sb.Label, setObjectName='lbl001', setText='Minimize App', setToolTip='Minimize the main application.')
		dh.ctxMenu.add(self.sb.Label, setObjectName='lbl002', setText='Maximize App', setToolTip='Restore the main application.')
		dh.ctxMenu.add(self.sb.Label, setObjectName='lbl003', setText='Close App', setToolTip='Close the main application.')

		cmb005 = self.sb.file.cmb005
		cmb005.ctxMenu.add('QPushButton', setObjectName='b001', setText='Last', setToolTip='Open the most recent file.')
		cmb005.addItems_(dict(zip(self.getRecentFiles(timestamp=True), self.getRecentFiles(timestamp=False))), "Recent Files", clear=True)

		cmb006 = self.sb.file.cmb006
		cmb006.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb001', setToolTip='Current project directory root.')
		cmb006.ctxMenu.add(self.sb.Label, setObjectName='lbl000', setText='Set', setToolTip='Set the project directory.')
		cmb006.ctxMenu.add(self.sb.Label, setObjectName='lbl004', setText='Root', setToolTip='Open the project directory.')
		cmb006.ctxMenu.cmb001.addItems_(self.getRecentProjects(), "Recent Projects", clear=True)

		tb000 = self.sb.file.draggable_header.ctxMenu.tb000
		tb000.ctxMenu.add('QCheckBox', setText='Wireframe', setObjectName='chk000', setToolTip='Set view to wireframe before save.')
		tb000.ctxMenu.add('QCheckBox', setText='Increment', setObjectName='chk001', setChecked=True, setToolTip='Append and increment a unique integer value.')
		tb000.ctxMenu.add('QCheckBox', setText='Quit', setObjectName='chk002', setToolTip='Quit after save.')


	def referenceSceneMenu(self, clear=False):
		'''
		'''
		try:
			if clear:
				del self._referenceSceneMenu
			return self._referenceSceneMenu

		except AttributeError as error:
			menu = self.sb.Menu(self.sb.file.lbl005)
			for i in self.getWorkspaceScenes(fullPath=True): #zip(self.getWorkspaceScenes(fullPath=False), self.getWorkspaceScenes(fullPath=True)):
				chk = menu.add(self.sb.CheckBox, setText=i)
				chk.toggled.connect(lambda state, scene=i: self.referenceScene(scene, not state))

				# if chk.sizeHint().width() > menu.sizeHint().width():
				# 	chk.setMinimumSize(chk.sizeHint().width(), chk.sizeHint().height())

			self.sb.setStyle(menu.childWidgets, style='dark')

			self._referenceSceneMenu = menu
			return self._referenceSceneMenu


	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.file.draggable_header


	def lbl005(self):
		'''Reference
		'''
		self.referenceSceneMenu().show()


	@Slots.hideMain
	def b001(self):
		'''Recent Files: Open Last
		'''
		# files = [file_ for file_ in (list(reversed(pm.optionVar(query='RecentFilesList')))) if "Autosave" not in file_]

		# force=True
		# if str(mel.eval("file -query -sceneName -shortName;")):
		# 	force=False #if sceneName, prompt user to save; else force open
		# pm.openFile(files[0], open=1, force=force)

		self.cmb005(index=1)


	@Slots.hideMain
	def b007(self):
		'''Import file
		'''
		self.cmb003(index=1)


	@Slots.hideMain
	def b008(self):
		'''Export Selection
		'''
		self.cmb004(index=1)









# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



# deprecated:


# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.init import Init



class Init_maya(Init, Slots_maya):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		try: #set the 'hud' textEdit to connect to the 'contruct_hud' method on show.
			self.sb.init.hud.shown.connect(self.construct_hud)

		except AttributeError as error: #(an inherited class)
			print (error)


	def construct_hud(self):
		'''Add current scene attributes to the hud lineEdit.
		Only those with relevant values will be displayed.
		'''
		hud = self.sb.init.hud

		try:
			selection = pm.ls(selection=1)
		except NameError:
			return

		if not selection:
			autoSaveState = pm.autoSave(q=True, enable=True)
			if not autoSaveState:
				hud.insertText('Autosave: <font style="color: {};">{}'.format('Green' if autoSaveState else 'Red', 'On' if autoSaveState else 'Off')) #symmetry axis

			sceneUnits = pm.currentUnit(query=1, fullName=1, linear=1)
			hud.insertText('Units: <font style="color: Yellow;">{}'.format(sceneUnits)) #symmetry axis

			symmetry = pm.symmetricModelling(query=1, symmetry=1);
			if symmetry:
				axis = pm.symmetricModelling(query=1, axis=1)
				hud.insertText('Symmetry Axis: <font style="color: Yellow;">{}'.format(axis.upper())) #symmetry axis

			xformConstraint = pm.xformConstraint(query=True, type=True)
			if xformConstraint=='none':
				xformConstraint=None
			if xformConstraint:
				hud.insertText('Xform Constraint: <font style="color: Yellow;">{}'.format(xformConstraint)) #transform constraits

		else:
			if pm.selectMode(query=1, object=1): #object mode:
				if pm.selectType(query=1, allObjects=1): #get object/s

					selectedObjects = pm.ls(selection=1)#, objectsOnly=1)
					numberOfSelected = len(selectedObjects)
					if numberOfSelected<11:
						name_and_type = ['<font style="color: Yellow;">{0}<font style="color: LightGray;">:{1}<br/>'.format(i.name(), pm.objectType(i)) for i in selectedObjects] #ie. ['pCube1:transform', 'pSphere1:transform']
						name_and_type_str = str(name_and_type).translate(str.maketrans('', '', ',[]\'')) #format as single string. remove brackets, single quotes, and commas.
					else:
						name_and_type_str = '' #if more than 10 objects selected, don't list each object.
					hud.insertText('Selected: <font style="color: Yellow;">{0}<br/>{1}'.format(numberOfSelected, name_and_type_str)) #currently selected objects by name and type.

					objectFaces = pm.polyEvaluate(selectedObjects, face=True)
					if type(objectFaces)==int:
						hud.insertText('Faces: <font style="color: Yellow;">{}'.format(objectFaces, ',d')) #add commas each 3 decimal places.

					objectTris = pm.polyEvaluate(selectedObjects, triangle=True)
					if type(objectTris)==int:
						hud.insertText('Tris: <font style="color: Yellow;">{}'.format(objectTris, ',d')) #add commas each 3 decimal places.

					objectUVs = pm.polyEvaluate(selectedObjects, uvcoord=True)
					if type(objectUVs)==int:
						hud.insertText('UVs: <font style="color: Yellow;">{}'.format(objectUVs, ',d')) #add commas each 3 decimal places.

			elif pm.selectMode(query=1, component=1): #component mode:
				if pm.selectType(query=1, vertex=1): #get vertex selection info
					type_ = 'Verts'
					num_selected = pm.polyEvaluate(vertexComponent=1)
					total_num = pm.polyEvaluate(selection, vertex=1)

				elif pm.selectType(query=1, edge=1): #get edge selection info
					type_ = 'Edges'
					num_selected = pm.polyEvaluate(edgeComponent=1)
					total_num = pm.polyEvaluate(selection, edge=1)

				elif pm.selectType(query=1, facet=1): #get face selection info
					type_ = 'Faces'
					num_selected = pm.polyEvaluate(faceComponent=1)
					total_num = pm.polyEvaluate(selection, face=1)

				elif pm.selectType(query=1, polymeshUV=1): #get uv selection info
					type_ = 'UVs'
					num_selected = pm.polyEvaluate(uvComponent=1)
					total_num = pm.polyEvaluate(selection, uvcoord=1)

				try:
					hud.insertText('Selected {}: <font style="color: Yellow;">{} <font style="color: LightGray;">/{}'.format(type_, num_selected, total_num)) #selected components
				except NameError:
					pass


		method = self.sb.prevCommand
		if method:
			hud.insertText('Prev Command: <font style="color: Yellow;">{}'.format(method.__doc__))  #get button text from last used command

		# prevUi = self.sb.prevUiName(omitLevel=[0,1,2])
		# hud.insertText('Prev UI: {}'.format(prevUi.replace('_', '').title())) #get the last level 3 ui name string.

		# prevCamera = self.sb.prevCamera(docString=True)
		# hud.insertText('Prev Camera: {}'.format(prevCamera)) #get the previously used camera.









#module name
print (__name__)
# ======================================================================
# Notes
# ======================================================================





#deprecated -------------------------------------




# !/usr/bin/python
# coding=utf-8
import sys

from PySide2 import QtCore, QtGui, QtWidgets


class OverlayFactoryFilter(QtCore.QObject):
	'''Handles paint events as an overlay on top of an existing widget.
	'''
	def eventFilter(self, widget, event):
		'''Relay events from the parent widget to the overlay.
		'''
		if not widget.isWidgetType():
			return False

		if event.type()==QtCore.QEvent.MouseButtonPress:
			self.mousePressEvent(event)

		elif event.type()==QtCore.QEvent.MouseButtonRelease:
			self.mouseReleaseEvent(event)

		elif event.type()==QtCore.QEvent.MouseMove:
			self.mouseMoveEvent(event)

		elif event.type()==QtCore.QEvent.MouseButtonDblClick:
			self.mouseDoubleClickEvent(event)

		elif event.type()==QtCore.QEvent.KeyPress:
			self.keyPressEvent(event)

		elif event.type()==QtCore.QEvent.KeyRelease:
			self.keyReleaseEvent(event)

		elif event.type()==QtCore.QEvent.Resize:
			if widget==self.parentWidget():
				self.resize(widget.size())

		elif event.type()==QtCore.QEvent.Show:
			self.raise_()

		return super().eventFilter(widget, event)



class Overlay(QtWidgets.QWidget, OverlayFactoryFilter):
	'''Handles paint events as an overlay on top of an existing widget.

	:Parameters:
		tcl (obj): tcl widget instance.
		antialiasing (bool): Set antialiasing for the tangent paint events.
	'''
	greyPen = QtGui.QPen(QtGui.QColor(115, 115, 115), 3, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
	blackPen = QtGui.QPen(QtGui.QColor(0, 0, 0), 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)

	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.

	def __init__(self, parent=None, antialiasing=False):
		super().__init__(parent)

		self.antialiasing = antialiasing
		self.drawEnabled = False

		self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
		self.setAttribute(QtCore.Qt.WA_NoSystemBackground) #takes a single arg
		self.setAttribute(QtCore.Qt.WA_TransparentForMouseEvents)
		self.setFocusPolicy(QtCore.Qt.NoFocus)

		self.painter = QtGui.QPainter() #Initialize self.painter

		if parent:
			parent.installEventFilter(self)


	@property
	def drawPath(self) -> list:
		'''
		'''
		try:
			return self._drawPath

		except AttributeError as error:
			self._drawPath = []
			return self._drawPath


	@drawPath.setter
	def drawPath(self, lst) -> None:
		'''
		'''
		self._drawPath = lst


	@property
	def drawPathStartPos(self):
		'''The starting widget ('returnArea') position stored in the drawPath.
		'''
		try:
			return self.drawPath[0][2]
		except IndexError as error:
			return None


	@property
	def drawPathEndPos(self):
		'''The ending widget position stored in the drawPath.
		Returns None if not found.
		'''
		try:
			return self.drawPath[-1][1]
		except IndexError as error:
			return None


	def getWidgetPos(self, widget):
		'''
		'''
		return next((wpos for w, wpos, cpos in self.drawPath if w==widget), None)


	def clearDrawPath(self):
		'''Reset the list of draw paths, while keeping the return point.
		'''
		del self.drawPath[1:]


	def addToDrawPath(self, w):
		'''Add a widget to the drawPath.
		It's current position, as well as the current cursor position, will be logged at the time of adding.
		'''
		w_pos = w.mapToGlobal(w.rect().center()) #the widget position before submenu change.
		self.drawPath.append((w, w_pos, QtGui.QCursor.pos())) #add the (<widget>, position) from the old ui to the path so that it can be re-created in the new ui (in the same position).
		self.removeFromPath(w)


	def removeFromPath(self, ui):
		'''Remove the last entry from the widget and draw paths for the given ui.

		:Parameters:
			ui (obj): The ui to remove.
		'''
		uis = [w.ui for w, wpos, cpos in self.drawPath[1:]]
		# print ([w.name for w, wpos, cpos in self.drawPath[1:]])
		if ui in uis:
			i = uis[::-1].index(ui) #reverse the list and get the index of the last occurrence of name.
			# print (i, -i-1, ui.name, [(w.ui.name, cpos) for w, wpos, cpos in self.drawPath]) #debug
			del self.drawPath[-i-1:]


	@property
	def returnArea(self) -> object:
		'''
		'''
		try:
			return self._returnArea

		except AttributeError as error:
			#create a return area widget. It's initial location does not matter, as it will later be re-created at cursor pos.
			w = QtWidgets.QPushButton(self)
			w.setObjectName('return_area')
			w.resize(45, 45)
			w.move(QtCore.QPoint(self.pos().x()-w.width(), self.pos().y()-w.height())) #center the widget.
			self._returnArea = w
			return self._returnArea


	def paintEvent(self, event):
		'''
		'''
		if not self.drawEnabled:
			return

		self.painter.begin(self)

		for i, (w, wpos, start_point) in enumerate(self.drawPath): #plot and draw the points in the drawPath list.

			start_point = self.mapFromGlobal(start_point)
			try:
				end_point = self.mapFromGlobal(self.drawPath[i+1][2])
			except IndexError as error:
				end_point = self.mouseMovePos #after the list points are drawn, plot the current end_point, controlled by the mouse move event.

			self.drawTangent(start_point, end_point)

		self.painter.end()


	def drawTangent(self, start_point, end_point, color=(115, 115, 115), background=(127, 127, 127, 0), ellipseSize=7):
		'''Draw a segment between two points.
		'''
		path = QtGui.QPainterPath()

		#create QColors from the given arguments.
		color = QtGui.QColor(*color)
		bgColor = QtGui.QColor(*background)
		#add ellipse if size is above 0.
		if ellipseSize:
			path.addEllipse(QtCore.QPointF(start_point), ellipseSize, ellipseSize)

		self.painter.fillRect(self.rect(), bgColor) #transparent overlay background.
		self.painter.setRenderHint(QtGui.QPainter.Antialiasing, self.antialiasing)
		self.painter.setBrush(color)
		self.painter.fillPath(path, color)
		self.painter.setPen(self.blackPen)
		self.painter.drawPath(path) #stroke
		if not end_point.isNull():
			self.painter.setPen(self.greyPen)
			self.painter.drawLine(start_point, end_point)
			# self.painter.drawEllipse(end_point, 5, 5)


	def keyReleaseEvent(self, event):
		'''
		'''
		if event.isAutoRepeat():
			return

		# modifiers = self.app.keyboardModifiers()
		del self.drawPath[:]
		self.update()


	def mousePressEvent(self, event):
		'''
		'''
		del self.drawPath[:]
		
		curPos = self.mapToGlobal(event.pos())

		self.drawPath.append([self.returnArea, curPos, curPos]) #maintain a list of widgets, their location, and cursor positions, as a path is plotted along the ui hierarchy.

		self.mouseMovePos = curPos
		self.update()


	def mouseReleaseEvent(self, event):
		'''
		'''
		self.drawEnabled = False
		self.update()


	def mouseMoveEvent(self, event):
		'''
		'''
		self.drawEnabled = True
		self.mouseMovePos = event.pos()
		self.update()


	def showEvent(self, event):
		'''
		'''

# --------------------------------------------------------------------------------------------






if __name__ == '__main__':
	import sys
	overlay = Overlay()

	sys.exit(overlay.app.exec_())


#module name
print (__name__)

# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# Deprecated: -----------------------------------

	# def getDrawPathWidget(self, index):
	# 	'''
	# 	'''
	# 	try:
	# 		return self.drawPath[index][0]
	# 	except IndexError as error:
	# 		return None


	# def getDrawPathWidgetPos(self, index):
	# 	'''
	# 	'''
	# 	try:
	# 		return self.drawPath[index][1]
	# 	except IndexError as error:
	# 		return None


	# def getDrawPathCursorPos(self, index):
	# 	'''
	# 	'''
	# 	try:
	# 		return self.drawPath[index][2]
	# 	except IndexError as error:
	# 		return None


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.uv import Uv



class Uv_max(Uv, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb000 = self.sb.uv.draggable_header.ctxMenu.cmb000
		items = ['UV Editor','UV Set Editor','UV Tool Kit','UV Linking: Texture-Centric','UV Linking: UV-Centric','UV Linking: Paint Effects/UV','UV Linking: Hair/UV']
		cmb000.addItems_(items, '3dsMax UV Editors')

		cmb002 = self.sb.uv.cmb002
		items = ['Flip U', 'Flip V', 'Align U Left', 'Align U Middle', 'Align U Right', 'Align V Top', 'Align V Middle', 'Align V Bottom', 'Linear Align']
		cmb002.addItems_(items, 'Transform:')

		tb000 = self.sb.uv.tb000
		tb000.ctxMenu.add('QSpinBox', setPrefix='Pre-Scale Mode: ', setObjectName='s009', setMinMax_='0-1 step1', setValue=1, setToolTip='Allow shell scaling during packing.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Pre-Rotate Mode: ', setObjectName='s010', setMinMax_='0-1 step1', setValue=1, setToolTip='Allow shell rotation during packing.')

		tb001 = self.sb.uv.tb001
		tb001.ctxMenu.add('QRadioButton', setText='Standard', setObjectName='chk000', setChecked=True, setToolTip='Create UV texture coordinates for the selected object or faces by automatically finding the best UV placement using simultanious projections from multiple planes.')
		tb001.ctxMenu.add('QCheckBox', setText='Scale Mode 1', setObjectName='chk001', setTristate=True, setChecked=True, setToolTip='0 - No scale is applied.<br>1 - Uniform scale to fit in unit square.<br>2 - Non proportional scale to fit in unit square.')
		tb001.ctxMenu.add('QRadioButton', setText='Seam Only', setObjectName='chk002', setToolTip='Cut seams only.')
		tb001.ctxMenu.add('QRadioButton', setText='Planar', setObjectName='chk003', setToolTip='Create UV texture coordinates for the current selection by using a planar projection shape.')
		tb001.ctxMenu.add('QRadioButton', setText='Cylindrical', setObjectName='chk004', setToolTip='Create UV texture coordinates for the current selection, using a cylidrical projection that gets wrapped around the mesh.<br>Best suited for completely enclosed cylidrical shapes with no holes or projections on the surface.')
		tb001.ctxMenu.add('QRadioButton', setText='Spherical', setObjectName='chk005', setToolTip='Create UV texture coordinates for the current selection, using a spherical projection that gets wrapped around the mesh.<br>Best suited for completely enclosed spherical shapes with no holes or projections on the surface.')
		tb001.ctxMenu.add('QRadioButton', setText='Normal-Based', setObjectName='chk006', setToolTip='Create UV texture coordinates for the current selection by creating a planar projection based on the average vector of it\'s face normals.')
		# tb001.ctxMenu.chk001.toggled.connect(lambda state: self.sb.toggleWidgets(tb001.ctxMenu, setUnChecked='chk002-3') if state==1 else None)


	@property
	def uvModifier(self):
		'''Get the UV modifier for the current object.
		If one doesn't exist, a UV modifier will be added to the selected object.

		:Return:
			(obj) uv modifier.
		'''
		selection = rt.selection
		if not selection:
			self.messageBox('Nothing selected.')

		mod = self.getModifier(selection[0], 'Unwrap_UVW', -1) #get/set the uv xform modifier.
		return mod


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.uv.draggable_header.ctxMenu.cmb000

		if index>0: #hide hotbox then perform operation
			self.sb.parent().hide()
			if index==1: #UV Editor
				maxEval('TextureViewWindow;') 
			elif index==2: #UV Set Editor
				maxEval('uvSetEditor;')
			elif index==3: #UV Tool Kit
				maxEval('toggleUVToolkit;')
			elif index==4: #UV Linking: Texture-Centric
				maxEval('textureCentricUvLinkingEditor;')
			elif index==5: #UV Linking: UV-Centric
				maxEval('uvCentricUvLinkingEditor;')
			elif index==6: #UV Linking: Paint Effects/UV
				maxEval('pfxUVLinkingEditor;')
			elif index==7: #UV Linking: Hair/UV
				maxEval('hairUVLinkingEditor;')
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Display
		'''
		cmb = self.sb.uv.cmb001


	def cmb002(self, index=-1):
		'''Transform
		'''
		cmb = self.sb.uv.cmb002

		if index>0:
			text = cmb.items[index]
			self.sb.parent().hide() #hide hotbox then perform operation
			if text=='Flip U':
				pm.polyFlipUV(flipType=0, local=1, usePivot=1, pivotU=0, pivotV=0)
			elif text=='Flip V':
				pm.polyFlipUV(flipType=1, local=1, usePivot=1, pivotU=0, pivotV=0)
			elif text=='Align U Left':
				pm.mel.performAlignUV('minU')
			elif text=='Align U Middle':
				pm.mel.performAlignUV('avgU')
			elif text=='Align U Right':
				pm.mel.performAlignUV('maxU')
			elif text=='Align U Top':
				pm.mel.performAlignUV('maxV')
			elif text=='Align U Middle':
				pm.mel.performAlignUV('avgV')
			elif text=='Align U Bottom':
				pm.mel.performAlignUV('minV')
			elif text=='Linear Align':
				pm.mel.performLinearAlignUV()
			cmb.setCurrentIndex(0)


	def chk014(self):
		'''Display: Checkered Pattern
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk014.isChecked()

		self.toggleMaterialOverride(checker=state)


	def chk015(self):
		'''Display: Borders
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk015.isChecked()

		borderWidth = pm.optionVar(query='displayPolyBorderEdgeSize')[1]
		borders = pm.polyOptions(displayMapBorder=state, sizeBorder=borderWidth)


	def chk016(self):
		'''Display: Distortion
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk016.isChecked()

		# actionMan.executeAction 2077580866 "40177"  -- Unwrap UVW: Show Edge Distortion
		mod = self.uv_uiModifier #get/set the uv modifier.

		mod.localDistortion = state
		self.messageBox('{0}{1}'.format('localDistortion:', state))


	def tb000(self, state=None):
		'''Pack UV's

		#pack command: Lets you pack the texture vertex elements so that they fit within a square space.
		# --method - 0 is a linear packing algorithm fast but not that efficient, 1 is a recursive algorithm slower but more efficient.
		# --spacing - the gap between cluster in percentage of the edge distance of the square
		# --normalize - determines whether the clusters will be fit to 0 to 1 space.
		# --rotate - determines whether a cluster will be rotated so it takes up less space.
		# --fillholes - determines whether smaller clusters will be put in the holes of the larger cluster.
		'''
		tb = self.sb.uv.tb000

		scale = tb.ctxMenu.s009.value()
		rotate = tb.ctxMenu.s010.value()

		obj = rt.selection[0]

		self.uv_uiModifier.pack(1, 0.01, scale, rotate, True) #(method, spacing, normalize, rotate, fillholes)


	@Slots_max.attr
	def tb001(self, state=None):
		'''Auto Unwrap
		'''
		tb = self.sb.uv.tb001

		standardUnwrap = tb.ctxMenu.chk000.isChecked()
		scaleMode = tb.ctxMenu.chk001.isChecked()
		seamOnly = tb.ctxMenu.chk002.isChecked()
		planarUnwrap = tb.ctxMenu.chk003.isChecked()
		cylindricalUnwrap = tb.ctxMenu.chk004.isChecked()
		sphericalUnwrap = tb.ctxMenu.chk005.isChecked()
		normalBasedUnwrap = tb.ctxMenu.chk006.isChecked()

		objects = rt.selection

		for obj in objects:
			if standardUnwrap:
				try:
					self.uv_uiModifier.FlattenBySmoothingGroup(scaleMode, False, 0.2)

				except Exception as error:
					print(error)


	def tb002(self, state=None):
		'''Stack
		'''
		tb = self.sb.uv.tb002

		orient = tb.ctxMenu.chk021.isChecked()
		stackSimilar = tb.ctxMenu.chk022.isChecked()
		tolerance = tb.ctxMenu.s000.value()
		sel = self.UvShellSelection() #assure the correct selection mask.

		if stackSimilar:
			pm.polyUVStackSimilarShells(sel, tolerance=tolerance)
		else:
			pm.mel.texStackShells([])
		if orient:
			pm.mel.texOrientShells()


	def tb003(self, state=None):
		'''Select By Type
		'''
		tb = self.sb.uv.tb003

		back_facing = tb.ctxMenu.chk008.isChecked()
		front_facing = tb.ctxMenu.chk009.isChecked()
		overlapping = tb.ctxMenu.chk010.isChecked()
		nonOverlapping = tb.ctxMenu.chk011.isChecked()
		textureBorders = tb.ctxMenu.chk012.isChecked()
		unmapped = tb.ctxMenu.chk013.isChecked()

		if back_facing:
			pm.mel.selectUVFaceOrientationComponents({}, 0, 2, 1)
		elif front_facing:
			pm.mel.selectUVFaceOrientationComponents({}, 0, 1, 1)
		elif overlapping:
			pm.mel.selectUVOverlappingComponents(1, 0)
		elif nonOverlapping:
			pm.mel.selectUVOverlappingComponents(0, 0)
		elif textureBorders:
			pm.mel.selectUVBorderComponents({}, "", 1)
		elif unmapped:
			pm.mel.selectUnmappedFaces()


	def tb004(self, state=None):
		'''Unfold
		'''
		tb = self.sb.uv.tb004

		optimize = self.tb.ctxMenu.chk017.isChecked()

		# if optimize:
		# 	# self.uv_uiModifier.
		# else:
		self.uv_uiModifier.relax(1, 0.01, True, True)


	def tb005(self, state=None):
		'''Straighten Uv
		'''
		tb = self.sb.uv.tb005

		u = tb.ctxMenu.chk018.isChecked()
		v = tb.ctxMenu.chk019.isChecked()
		angle = tb.ctxMenu.s001.value()
		straightenShell = tb.ctxMenu.chk020.isChecked()

		# if u:
		# 	contraint = 'U'
		# if v:
		# 	constaint = 'V' if not u else 'UV'

		self.uv_uiModifier.Straighten()

		# if straightenShell:
		# 	pm.mel.texStraightenShell()


	def tb006(self, state=None):
		'''Distribute
		'''
		tb = self.sb.uv.tb006

		u = tb.ctxMenu.chk023.isChecked()
		v = tb.ctxMenu.chk024.isChecked()
		
		if u:
			pm.mel.texDistributeShells(0, 0, "right", []) #'left', 'right'
		if v:
			pm.mel.texDistributeShells(0, 0, "down", []) #'up', 'down'


	def tb008(self, state=None):
		'''Transfer UV's
		'''
		tb = self.sb.uv.tb008

		toSimilar = tb.ctxMenu.chk025.isChecked()
		similarTol = tb.ctxMenu.s013.value()
		deleteConstHist = tb.ctxMenu.chk026.isChecked()

		frm, *to = pm.ls(orderedSelection=1, flatten=1)
		if toSimilar:
			to = 'similar'
		elif not to:
			return self.messageBox('<b>Nothing selected.</b><br>The operation requires the selection of two polygon objects.')

		self.transferUVs(frm, to, tol=similarTol, deleteConstHist=deleteConstHist)


	def b001(self):
		'''Create UV Snapshot
		'''
		pass


	def b002(self):
		'''Stack Shells
		'''
		pm.mel.texStackShells()
		# pm.mel.texOrientShells()


	def b003(self):
		'''Get texel density.
		'''
		density = pm.mel.texGetTexelDensity(self.getMapSize())
		self.sb.uv.s003.setValue(density)


	def b004(self):
		'''Set Texel Density
		'''	
		density = self.sb.uv.s003.value()
		mapSize = self.getMapSize()
		pm.mel.texSetTexelDensity(density, mapSize)


	def b005(self):
		'''Cut Uv'S
		'''
		self.uv_uiModifier.breakSelected()


	def b011(self):
		'''Sew Uv'S
		'''
		self.uv_uiModifier.stitchVerts(True, 1.0) #(align, bias) --Bias of 0.0 the vertices will move to the source and 1.0 they will move to the target. 


	def moveSelectedToUvSpace(self, u, v, relative=True):
		'''Move sny selected objects to the given u and v coordinates.

		:Parameters:
			u (int): u coordinate.
			v (int): v coordinate.
			relative (bool): Move relative or absolute.
		'''
		mod = self.uv_uiModifier

		pm.polyEditUV(sel, u=u, v=v, relative=relative)


	def flipUV(self, objects=None):
		''''''
		u = 1
		v = 0
		w = 0

		if not objects:
			objects = rt.selection

		for obj in objects:
			try:
				uv = self.uv_uiModifier #get/set the uv xform modifier.
				uv.U_Flip = u
				uv.V_Flip = v
				uv.W_Flip = w

			except Exception as error:
				print(error)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

#apply uv map
# maxEval('modPanel.addModToSelection (Uvwmap ()) ui:on')


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.subdivision import Subdivision



class Subdivision_max(Subdivision, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		#Set 3ds Max specific naming
		self.sb.subdivision.gb000.setTitle('TurboSmooth')
		self.sb.subdivision.s000.setPrefix('Iterations:  ')
		self.sb.subdivision.s001.setPrefix('RenderIters: ')
		self.sb.subdivision.s000.setValue(0)
		self.sb.subdivision.s001.setValue(0)

		ctx = self.sb.subdivision.draggable_header.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Max Subdivision Editiors.')
			# ctx.add(self.sb.ComboBox, setObjectName='cmb001', setToolTip='Smooth Proxy.')
			# ctx.add(self.sb.ComboBox, setObjectName='cmb002', setToolTip='Maya Subdivision Operations.')

		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb000
		items = ['TurboSmooth','TurboSmooth Pro','OpenSubDiv','Subdivide','Subdivide (WSM)','MeshSmooth','Optimize','Pro Optimizer','Add Divisions']
		cmb.addItems_(items, '3dsMax Subdivision Modifiers')

		# cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb001
		# items = ['Create Subdiv Proxy','Remove Subdiv Proxy Mirror','Crease Tool','Toggle Subdiv Proxy Display', 'Both Proxy and Subdiv Display']
		# cmb.addItems_(items, 'Smooth Proxy')

		# cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb002
		# items = ['Reduce Polygons','Add Divisions','Smooth']
		# cmb.addItems_(items, 'Maya Subdivision Operations')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='TurboSmooth':
				mod = rt.TurboSmooth()
				mod.iterations = 0
				mod.renderIterations = 1
				mod.useRenderIterations = True
				mod.explicitNormals = True
				mod.sepByMats = True
				rt.modpanel.addModToSelection(mod)

			if text=='TurboSmooth Pro':
				mod = TurboSmooth_Pro()
				mod.iterations = 0
				mod.renderIterations = 1
				mod.useRenderIterations = True
				mod.explicitNormals = True
				mod.visualizeCreases = True
				mod.smmoothCorners = False
				mod.creaseSmoothingGroups = True
				mod.creaseMaterials = True
				rt.modpanel.addModToSelection(mod)

			if text=='OpenSubDiv':
				mod = OpenSubdiv()
				mod.iterations = 0
				mod.renderIterations = 1
				mod.useRenderIterations = True
				rt.modpanel.addModToSelection(mod)

			if text=='Subdivide':
				mod = subdivide()
				mod.size = 0.075
				rt.modpanel.addModToSelection(mod)

			if text=='Subdivide (WSM)':
				mod = subdivideSpacewarpModifier()
				mod.size = 40
				rt.modpanel.addModToSelection(mod)

			if text=='MeshSmooth':
				mod = meshsmooth()
				mod.iterations = 0
				mod.renderIterations = 1
				mod.useRenderIterations = True
				mod.useRenderSmoothness = True
				rt.modpanel.addModToSelection(mod)

			if text=='Optimize':
				mod = optimize()
				rt.modpanel.addModToSelection(mod)

			if text=='Pro optimizer':
				mod = ProOptimizer()
				rt.modpanel.addModToSelection(mod)

			if text=='Add Divisions':
				maxEval('''
				Try (
					local A = modPanel.getCurrentObject()
					A.popupDialog #Tessellate
				)
				Catch ()
				''')
			cmb.setCurrentIndex(0)


	def s000(self, value=None):
		'''Division Level
		'''
		value = self.sb.subdivision.s000.getValue()

		geometry = rt.selection

		for obj in geometry:
			try:
				obj.modifiers['TurboSmooth'].iterations = value
			except: pass


	def s001(self, value=None):
		'''Tesselation Level
		'''
		value = self.sb.subdivision.s001.getValue()

		geometry = rt.selection

		for obj in geometry:
			try:
				obj.modifiers['TurboSmooth'].renderIterations = value
			except: pass


	def b005(self):
		'''Reduce
		'''
		mel.eval("ReducePolygon;")


	def b008(self):
		'''Add Divisions - Subdivide Mesh
		'''
		maxEval('macros.run \"Modifiers\" \"Tessellate\"')


	def b009(self):
		'''Smooth
		'''
		maxEval('macros.run \"Modifiers\" \"Smooth\"')


	def b010(self):
		''''''
		pass


	def b011(self):
		'''Convert Smooth Preview
		'''
		#convert smooth mesh preview to polygons
		geometry = rt.selection

		if not len(geometry):
			geometry = rt.geometry

		for obj in geometry:
			try:
				renderIters = obj.modifiers['TurboSmooth'].renderIterations
				obj.modifiers['TurboSmooth'].iterations = renderIters

				modifiers = [mod.name for mod in obj.modifiers]
				if modifiers:
					index = modifiers.index('TurboSmooth') +1
				rt.maxOps.CollapseNodeTo(obj, index, False)
			except: pass









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.normals import Normals



class Normals_max(Normals, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		self.sb.normals.b003.setText('Hard Edge Display')

		cmb = self.sb.normals.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb.addItems_(items, '')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.normals.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Display Face Normals
		'''
		tb = self.sb.normals.tb000

		size = float(tb.ctxMenu.s001.value())

		self.messageBox('No 3ds Version.')
		tb.setDisabled(True)
		# # state = pm.polyOptions (query=True, displayNormal=True)
		# state = ptk.cycle([1,2,3,0], 'displayNormals')
		# if state ==0: #off
		# 	pm.polyOptions (displayNormal=0, sizeNormal=0)
		# 	pm.polyOptions (displayTangent=False)
		# 	return 'Normals Display <hl>Off</hl>.'

		# elif state ==1: #facet
		# 	pm.polyOptions (displayNormal=1, facet=True, sizeNormal=size)
		# 	pm.polyOptions (displayTangent=False)
		# 	return '<hl>Facet</hl> Normals Display <hl>On</hl>.'

		# elif state ==2: #Vertex
		# 	pm.polyOptions (displayNormal=1, point=True, sizeNormal=size)
		# 	pm.polyOptions (displayTangent=False)
		# 	return '<hl>Vertex</hl> Normals Display <hl>On</hl>.'

		# elif state ==3: #tangent
		# 	pm.polyOptions (displayTangent=True)
		# 	pm.polyOptions (displayNormal=0)
		# 	return '<hl>Tangent</hl> Display <hl>On</hl>.'


	def tb001(self, state=None):
		'''Harden Edge Normals
		'''
		tb = self.sb.normals.tb001

		maxEval('$.EditablePoly.makeHardEdges 1')

		# hardAngle = tb.ctxMenu.s002.value()
		# hardenCreased = tb.ctxMenu.chk001.isChecked()
		# hardenUvBorders = tb.ctxMenu.chk002.isChecked()
		# softenOther = tb.ctxMenu.chk000.isChecked()

		# objects = rt.selection

		# for obj in objects:
			# selection = pm.ls(obj, sl=True, l=True)
			# selEdges = pm.ls(pm.polyListComponentConversion(selection, toEdge=1), flatten=1)
			# allEdges = edges = pm.ls(pm.polyListComponentConversion(obj, toEdge=1), flatten=1)

			# if hardenCreased:
			# 	creasedEdges = Normals.getCreasedEdges(allEdges)
			# 	selEdges = selEdges + creasedEdges if not selEdges==allEdges else creasedEdges

			# if hardenUvBorders:
			# 	uv_border_edges = Slots_max.getUvShellBorderEdges(selection)
			# 	selEdges = selEdges + uv_border_edges if not selEdges==allEdges else uv_border_edges

			# obj.EditablePoly.makeHardEdges(1) #set hard edges.

			# if softenOther:
			# 	invEdges = [e for e in allEdges if e not in selEdges]
			# 	pm.polySoftEdge(invEdges, angle=180, constructionHistory=0) #set soft edges.

			# rt.select(selEdges)


	@Slots.hideMain
	def tb002(self, state=None):
		'''Set Normal By Angle
		'''
		tb = self.sb.normals.tb002

		normalAngle = str(tb.ctxMenu.s000.value())
		subObjectLevel = rt.subObjectLevel


		if subObjectLevel==4: #smooth selected faces
			for obj in rt.selection:
				obj.autoSmoothThreshold = normalAngle
				# faceSelection = rt.polyop.getFaceSelection(obj)
				rt.polyop.autoSmooth(obj)
				rt.update(obj)

		else: #smooth entire mesh
			mod = rt.Smooth()
			mod.autoSmooth = True
			mod.threshold = normalAngle

			for obj in rt.selection:
				rt.modPanel.setCurrentObject(obj.baseObject)
				rt.modPanel.addModToSelection (mod)
				index = [mod for mod in obj.modifiers].index(mod)+1 #add one to convert index from python to maxscript
				rt.maxOps.CollapseNodeTo(obj, index, False)


	def tb003(self, state=None):
		'''Lock/Unlock Vertex Normals
		'''
		tb = self.sb.normals.tb003

		print ('# Error: No 3ds Version of this command yet. #')
		tb.setDisabled(True)
		# all_ = tb.ctxMenu.chk001.isChecked()
		# state = tb.ctxMenu.chk002.isChecked() #pm.polyNormalPerVertex(vertex, query=1, freezeNormal=1)
		# selection = pm.ls (selection=1, objectsOnly=1)
		# maskObject = pm.selectMode (query=1, object=1)
		# maskVertex = pm.selectType (query=1, vertex=1)

		# if len(selection)>0:
		# 	if (all_ and maskVertex) or maskObject:
		# 		for obj in selection:
		# 			count = pm.polyEvaluate(obj, vertex=1) #get number of vertices
		# 			vertices = [vertices.append(str(obj) + ".vtx ["+str(num)+"]") for num in range(count)] #geometry.vtx[0]
		# 			for vertex in vertices:
		# 				if not state:
		# 					pm.polyNormalPerVertex(vertex, unFreezeNormal=1)
		# 				else:
		# 					pm.polyNormalPerVertex(vertex, freezeNormal=1)
		# 			if not state:
		# 				self.mtk.viewportMessage("Normals <hl>UnLocked</hl>.")
		# 			else:
		# 				self.mtk.viewportMessage("Normals <hl>Locked</hl>.")
		# 	elif maskVertex and not maskObject:
		# 		if not state:
		# 			pm.polyNormalPerVertex(unFreezeNormal=1)
		# 			self.mtk.viewportMessage("Normals <hl>UnLocked</hl>.")
		# 		else:
		# 			pm.polyNormalPerVertex(freezeNormal=1)
		# 			self.mtk.viewportMessage("Normals <hl>Locked</hl>.")
		# 	else:
		# 		return 'Warning: Selection must be object or vertex.'
		# else:
		# 	return 'Warning: No object selected.'


	def tb004(self, state=None):
		'''Average Normals
		'''
		tb = self.sb.normals.tb004

		byUvShell = tb.ctxMenu.chk003.isChecked()

		if byUvShell:
			print ('# Error: No 3ds Version of this flag yet. #')
			sets_ = Slots_max.getUvShellSets(obj)
			for set_ in sets_:
				pm.polySetToFaceNormal(set_)
				pm.polyAverageNormal(set_)
		else:
			maxEval('macros.run "PolyTools" "SmoothSelection"')


	@Slots.hideMain
	def b001(self):
		'''Soften Edge Normal
		'''
		maxEval('$.EditablePoly.makeSmoothEdges 1')


	def b003(self):
		'''Soft Edge Display
		'''
		for obj in rt.selection:
			state = obj.hardedgedisplay
			obj.hardedgedisplay = not state


	def b005(self):
		'''Adjust Vertex Normals
		'''
		maxEval('bgAdjustVertexNormalsWin;')


	def b006(self):
		'''Set To Face
		'''
		maxEval('macros.run "PolyTools" "HardSelection"')


	def b010(self):
		'''Reverse Normals
		'''
		for obj in rt.selection:		
			rt.modPanel.setCurrentObject(obj.baseObject)
			
			mod = rt.Normalmodifier()
			mod.flip = True
			
			rt.modpanel.addModToSelection(mod)
			
			index = rt.modPanel.getModifierIndex(obj, mod)
			rt.maxOps.CollapseNodeTo(obj, index, False)

		







#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.materials import Materials



class Materials_blender(Materials, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Materials.__init__(self, *args, **kwargs)

		self.randomMat = None

		dh = self.sb.materials.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Material Editors')
		dh.ctxMenu.add(self.sb.Label, setText='Material Attributes', setObjectName='lbl004', setToolTip='Show the material attributes in the attribute editor.')

		cmb000 = self.sb.materials.draggable_header.ctxMenu.cmb000
		items = []
		cmb000.addItems_(items, 'Material Editors')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.materials.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Hypershade':
				mel.eval('HypershadeWindow;')
			cmb.setCurrentIndex(0)


	def cmb002(self, index=-1):
		'''Material list

		:Parameters:
			index (int): parameter on activated, currentIndexChanged, and highlighted signals.
		'''
		cmb = self.sb.materials.cmb002
		b = self.sb.materials_submenu.b003

		mode = cmb.ctxMenu.cmb001.currentText()
		if mode=='Scene Materials':
			materials = self.getSceneMaterials(exc=['standardSurface'])

		elif mode=='ID Map Materials':
			materials = self.getSceneMaterials(inc=['ID_*'])

		if mode=='Favorite Materials':
			fav_materials = self.getFavoriteMaterials()
			currentMats = {matName:matName for matName in sorted(list(set(fav_materials)))}
		else:
			currentMats = {mat.name():mat for mat in sorted(list(set(materials))) if hasattr(mat, 'name')} 

		cmb.addItems_(currentMats, clear=True)

		#create and set icons with color swatch
		for i, mat in enumerate(cmb.items):
			icon = self.getColorSwatchIcon(mat)
			cmb.setItemIcon(i, icon) if icon else None

		#set submenu assign material button attributes
		b.setText('Assign '+cmb.currentText())
		icon = self.getColorSwatchIcon(cmb.currentText(), [15, 15])
		b.setIcon(icon) if icon else None
		b.setMinimumWidth(b.minimumSizeHint().width()+25)
		b.setVisible(True if cmb.currentText() else False)


	def tb000(self, state=None):
		'''Select By Material Id
		'''
		tb = self.sb.materials.tb000

		mat = self.sb.materials.cmb002.currentData()
		if not mat:
			return 'Error: No Material Selection.'

		shell = tb.ctxMenu.chk005.isChecked() #Select by material: shell
		invert = tb.ctxMenu.chk006.isChecked() #Select by material: invert
		allObjects = tb.ctxMenu.chk003.isChecked() #Search all scene objects

		objects = pm.ls(sl=1, objectsOnly=1) if not allObjects else None

		self.selectByMaterialID(mat, objects, shell=shell, invert=invert)


	def tb002(self, state=None):
		'''Assign Material
		'''
		tb = self.sb.materials.tb002

		selection = pm.ls(selection=1, flatten=1)
		if not selection:
			return 'Error: No renderable object is selected for assignment.'

		assignCurrent = tb.ctxMenu.chk007.isChecked()
		assignRandom = tb.ctxMenu.chk008.isChecked()
		assignNew = tb.ctxMenu.chk009.isChecked()

		if assignCurrent: #Assign current mat
			mat = self.sb.materials.cmb002.currentData()
			if isinstance(mat, str): #new mat type as a string:
				self.assignMaterial(selection, pm.createNode(mat))
			else: #existing mat object:
				self.assignMaterial(selection, mat)

		elif assignRandom: #Assign New random mat ID
			mat = self.createRandomMaterial(prefix='ID_')
			self.assignMaterial(selection, mat)

			self.randomMat = mat

			self.cmb002() #refresh the materials list comboBox
			self.sb.materials.cmb002.setCurrentItem(mat.name()) #set the combobox index to the new mat #self.cmb002.setCurrentIndex(self.cmb002.findText(name))

		elif assignNew: #Assign New Material
			mel.eval('buildObjectMenuItemsNow "MainPane|viewPanes|modelPanel4|modelPanel4|modelPanel4|modelPanel4ObjectPop";')
			mel.eval('createAssignNewMaterialTreeLister "";')


	def lbl000(self):
		'''Open material in editor
		'''
		try:
			mat = self.sb.materials.cmb002.currentData() #get the mat obj from cmb002
			pm.select(mat)
		except:
			return 'Error: No stored material or no valid object selected.'

		pm.mel.HypershadeWindow() #open the hypershade editor


	def lbl001(self, setEditable=True):
		'''Rename Material: Set cmb002 as editable and disable wgts.
		'''
		cmb = self.sb.materials.cmb002

		if setEditable:
			self._mat = self.sb.materials.cmb002.currentData()
			cmb.setEditable(True)
			self.sb.toggleWidgets(self.sb.materials, setDisabled='b002,lbl000,tb000,tb002')
		else:
			mat = self._mat
			newMatName = cmb.currentText()
			self.renameMaterial(mat, newMatName)
			cmb.setEditable(False)
			self.sb.toggleWidgets(self.sb.materials, setEnabled='b002,lbl000,tb000,tb002')


	def lbl002(self):
		'''Delete Material
		'''
		mat = self.sb.materials.cmb002.currentData()
		mat = pm.delete(mat)

		index = self.sb.materials.cmb002.currentIndex()
		self.sb.materials.cmb002.setItemText(index, 'None') #self.sb.materials.cmb002.removeItem(index)


	def lbl003(self):
		'''Delete Unused Materials
		'''
		mel.eval('hyperShadePanelMenuCommand("hyperShadePanel1", "deleteUnusedNodes");')
		self.cmb002() #refresh the materials list comboBox


	def lbl004(self):
		'''Material Attributes: Show Material Attributes in the Attribute Editor.
		'''
		mat = self.sb.materials.cmb002.currentData()
		try:
			mel.eval('showSG '+mat.name())

		except Exception as error:
			print (error)


	def b000(self):
		'''Material List: Delete
		'''
		self.lbl002()


	def b001(self):
		'''Material List: Edit
		'''
		self.lbl000()


	def b002(self):
		'''Set Material: Set the Currently Selected Material as the currentMaterial.
		'''
		selection = pm.ls(selection=1)
		if not selection:
			return 'Error: Nothing selected.'

		mat = self.getMaterial()

		self.sb.materials.cmb002.ctxMenu.cmb001.setCurrentIndex(0) #set the combobox to show all scene materials
		self.cmb002() #refresh the materials list comboBox
		self.sb.materials.cmb002.setCurrentItem(mat.name())


	def getColorSwatchIcon(self, mat, size=[20, 20]):
		'''Get an icon with a color fill matching the given materials RBG value.

		:Parameters:
			mat (obj)(str): The material or the material's name.
			size (list): Desired icon size.

		:Return:
			(obj) pixmap icon.
		'''
		try:
			matName = mat.name() if not isinstance(mat, (str)) else mat #get the string name if a mat object is given.
			r = int(pm.getAttr(matName+'.colorR')*255) #convert from 0-1 to 0-255 value and then to an integer
			g = int(pm.getAttr(matName+'.colorG')*255)
			b = int(pm.getAttr(matName+'.colorB')*255)
			pixmap = QtGui.QPixmap(size[0],size[1])
			pixmap.fill(QtGui.QColor.fromRgb(r, g, b))

			return QtGui.QIcon(pixmap)

		except AttributeError:
			pass


	def renameMaterial(self, mat, newMatName):
		'''Rename Material
		'''
		cmb = self.sb.materials.cmb002 #scene materials

		curMatName = mat.name()
		if curMatName!=newMatName:
			cmb.setItemText(cmb.currentIndex(), newMatName)
			try:
				print (curMatName, newMatName)
				pm.rename(curMatName, newMatName)

			except RuntimeError as error:
				cmb.setItemText(cmb.currentIndex(), str(error).strip('\n'))


	def selectByMaterialID(self, material=None, objects=None, shell=False, invert=False):
		'''Select By Material Id
	
		material (obj): The material to search and select for.
		objects (list): Faces or mesh objects as a list. If no objects are given, all geometry in the scene will be searched.
		shell (bool): Select the entire shell.
		invert (bool): Invert the final selection.R

		#ex call:
		selectByMaterialID(material)
		'''
		if pm.nodeType(material)=='VRayMultiSubTex': #if not a multimaterial
			return 'Error: If material is a multimaterial, select a submaterial.'

		if not material:
			if not pm.ls(sl=1):
				return 'Error: Nothing selected. Select an object face, or choose the option: current material.'
			material = self.getMaterial()

		pm.select(material)
		pm.hyperShade(objects='') #select all with material. "" defaults to currently selected materials.

		if objects:
			[pm.select(i, deselect=1) for i in pm.ls(sl=1) if i.split('.')[0] not in objects]

		faces = pm.filterExpand(selectionMask=34, expand=1)
		transforms = pm.listRelatives(faces, p=True) #[node.replace('Shape','') for node in pm.ls(sl=1, objectsOnly=1, visible=1)] #get transform node name from shape node

		if shell or invert: #deselect so that the selection can be modified.
			pm.select(faces, deselect=1)

		if shell:
			for shell in transforms:
				pm.select(shell, add=1)
		
		if invert:
			for shell in transforms:
				allFaces = [shell+".f["+str(num)+"]" for num in range(pm.polyEvaluate (shell, face=1))] #create a list of all faces per shell
				pm.select(list(set(allFaces)-set(faces)), add=1) #get inverse of previously selected faces from allFaces


	def getSceneMaterials(self, inc=[], exc=[]):
		'''Get all materials from the current scene.

		:Parameters:
			inc (str)(int)(obj)(list): The objects(s) to include.
					supports using the '*' operator: startswith*, *endswith, *contains*
					Will include all items that satisfy ANY of the given search terms.
					meaning: '*.png' and '*Normal*' returns all strings ending in '.png' AND all 
					strings containing 'Normal'. NOT strings satisfying both terms.
			exc (str)(int)(obj)(list): The objects(s) to exclude. Similar to include.
					exlude take precidence over include.
		:Return:
			(list) materials.
		'''
		matList = pm.ls(mat=1, flatten=1)

		#convert to dictionary to filter material names and types.
		d = {m.name():pm.nodeType(m) for m in matList}
		filtered = Iter.filterDict(d, inc, exc, keys=True, values=True)

		#use the filtered results to reconstruct a filtered list of actual materials.
		return [m for m in matList if m.name() in filtered]


	def getFavoriteMaterials(self):
		'''Get Maya Favorite Materials List.

		:Return:
			(list) materials.
		'''
		import maya.app.general.tlfavorites as _fav, os.path

		path = os.path.expandvars(r"%USERPROFILE%/Documents/maya/2022/prefs/renderNodeTypeFavorites")
		renderNodeTypeFavorites = _fav.readFavorites(path)
		materials = [i for i in renderNodeTypeFavorites if '/' not in i]
		del _fav

		return materials


	def getMaterial(self, obj=''):
		'''Get the material from the selected face.

		:Parameters:
			(str)(obj): The obj with the material.

		:Return:
			(list) material
		'''
		pm.hyperShade(obj, shaderNetworksSelectMaterialNodes=1) #selects the material node 
		mats = pm.ls(selection=1, materials=1) #now add the selected node to a variable

		return mats[0]


	def createRandomMaterial(self, name='', prefix=''):
		'''Creates a random material.

		:Parameters:
			name (str): material name.
			prefix (str): Optional string to be appended to the beginning of the name.

		:Return:
			(obj) material.
		'''
		import random
		rgb = [random.randint(0, 255) for _ in range(3)] #generate a list containing 3 values between 0-255

		name = '{}{}_{}_{}_{}'.format(prefix, name, str(rgb[0]), str(rgb[1]), str(rgb[2]))

		#create shader
		mat = pm.shadingNode('lambert', asShader=1, name=name)
		#convert RGB to 0-1 values and assign to shader
		convertedRGB = [round(float(v)/255, 3) for v in rgb]
		pm.setAttr(name+'.color', convertedRGB)
		#assign to selected geometry
		# pm.select(selection) #initial selection is lost upon node creation
		# pm.hyperShade(assign=mat)

		return mat


	@Slots_blender.undoChunk
	def assignMaterial(self, objects, mat):
		'''Assign Material

		objects (list): Faces or mesh objects as a list.
		material (obj): The material to search and select for.
		'''
		if not mat:
			return 'Error: Material Not Assigned. No material given.'

		try: #if the mat is a not a known type; try and create the material.
			pm.nodeType(mat)
		except:
			mat = pm.shadingNode(mat, asShader=1)

		# pm.undoInfo(openChunk=1)
		for obj in pm.ls(objects):
			pm.select(obj) #hyperShade works more reliably with an explicit selection.
			pm.hyperShade(obj, assign=mat)
		# pm.undoInfo(closeChunk=1)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

#depricated


	# @property
	# def currentMat(self):
	# 	'''Get the current material using the current index of the materials combobox.
	# 	'''
	# 	text = self.sb.materials.cmb002.currentText()

	# 	try:
	# 		result = self.currentMats[text]
	# 	except:
	# 		result = None

	# 	return result

# elif storedMaterial:
# 	mat = self.currentMat
# 	if not mat:
# 		cmb.addItems_(['Stored Material: None'])
# 		return

# 	matName = mat.name()

# 	if pm.nodeType(mat)=='VRayMultiSubTex':
# 		subMaterials = pm.hyperShade(mat, listUpstreamShaderNodes=1) #get any connected submaterials
# 		subMatNames = [s.name() for s in subMaterials if s is not None]
# 	else:
# 		subMatNames=[]

# 	contents = cmb.addItems_(subMatNames, matName)

# 	if index is None:
# 		index = cmb.currentIndex()
# 	if index!=0:
# 		self.currentMat = subMaterials[index-1]
# 	else:
# 		self.currentMat = mat

# def cmb000(self, index=-1):
	# 	'''
	# 	Existing Materials

	# 	'''
	# 	cmb = self.sb.materials.draggable_header.ctxMenu.cmb000

	# 	mats = [m for m in pm.ls(materials=1)]
	# 	matNames = [m.name() for m in mats]

	# 	contents = cmb.addItems_(matNames, "Scene Materials")

	# 	if index is None:
	# 		index = cmb.currentIndex()
	# 	if index!=0:
	# 		print contents[index]
			
	# 		self.currentMat = mats[index-1] #store material
	# 		self.cmb002() #refresh combobox

	# 		cmb.setCurrentIndex(0)


	



#assign random
	# mel.eval('''
# 		string $selection[] = `ls -selection`;

# 		int $d = 2; //decimal places to round to
# 		$r = rand (0,1);
# 		$r = trunc($r*`pow 10 $d`+0.5)/`pow 10 $d`;
# 		$g = rand (0,1);
# 		$g = trunc($g*`pow 10 $d`+0.5)/`pow 10 $d`;
# 		$b = rand (0,1);
# 		$b = trunc($b*`pow 10 $d`+0.5)/`pow 10 $d`;

# 		string $rgb = ("_"+$r+"_"+$g+"_"+$b);
# 		$rgb = substituteAllString($rgb, "0.", "");

# 		$name = ("ID_"+$rgb);

# 		string $ID_ = `shadingNode -asShader lambert -name $name`;
# 		setAttr ($name + ".colorR") $r;
# 		setAttr ($name + ".colorG") $g;
# 		setAttr ($name + ".colorB") $b;

# 		for ($object in $selection)
# 			{
# 			select $object;
# 			hyperShade -assign $ID_;
# 			}
# 		 ''')

#re-assign random
	# mel.eval('''
		# string $objList[] = `ls -selection -flatten`;
		# $material = `hyperShade -shaderNetworksSelectMaterialNodes ""`;
		# string $matList[] = `ls -selection -flatten`;

		# hyperShade -objects $material;
		# string $selection[] = `ls -selection`;
		# //delete the old material and shader group nodes
		# for($i=0; $i<size($matList); $i++)
		# 	{
		# 	string $matSGplug[] = `connectionInfo -dfs ($matList[$i] + ".outColor")`;
		# 	$SGList[$i] = `match "^[^\.]*" $matSGplug[0]`;
		# 	print $matList; print $SGList;
		# 	delete $matList[$i];
		# 	delete $SGList[$i];
		# 	}
		# //create new random material
		# int $d = 2; //decimal places to round to
		# $r = rand (0,1);
		# $r = trunc($r*`pow 10 $d`+0.5)/`pow 10 $d`;
		# $g = rand (0,1);
		# $g = trunc($g*`pow 10 $d`+0.5)/`pow 10 $d`;
		# $b = rand (0,1);
		# $b = trunc($b*`pow 10 $d`+0.5)/`pow 10 $d`;

		# string $rgb = ("_"+$r+"_"+$g+"_"+$b+"");
		# $rgb = substituteAllString($rgb, "0.", "");

		# $name = ("ID_"+$rgb);

		# string $ID_ = `shadingNode -asShader lambert -name $name`;
		# setAttr ($name + ".colorR") $r;
		# setAttr ($name + ".colorG") $g;
		# setAttr ($name + ".colorB") $b;

		# for ($object in $selection)
		# 	{
		# 	select $object;
		# 	hyperShade -assign $ID_;
		# 	}
		# ''')


# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.uv import Uv



class Uv_blender(Uv, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Uv.__init__(self, *args, **kwargs)

		cmb000 = self.sb.uv.draggable_header.ctxMenu.cmb000
		items = []
		cmb000.addItems_(items, 'UV Editors')

		cmb001 = self.sb.uv.cmb001
		# panel = pm.getPanel(scriptType='polyTexturePlacementPanel')
		# cmb001.menu_.chk014.setChecked(pm.textureWindow(panel, displayCheckered=1, query=1)) #checkered state
		# cmb001.menu_.chk015.setChecked(True if pm.polyOptions(query=1, displayMapBorder=1) else False) #borders state
		# cmb001.menu_.chk016.setChecked(pm.textureWindow(panel, query=1, displayDistortion=1)) #distortion state

		cmb002 = self.sb.uv.cmb002
		items = []
		cmb002.addItems_(items, 'Transform:')

		tb000 = self.sb.uv.tb000
		tb000.ctxMenu.add('QSpinBox', setPrefix='Pre-Scale Mode: ', setObjectName='s009', setMinMax_='0-2 step1', setValue=1, setToolTip='Allow shell scaling during packing.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Pre-Rotate Mode: ', setObjectName='s010', setMinMax_='0-2 step1', setValue=1, setToolTip='Allow shell rotation during packing.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='Rotate Step: ', setObjectName='s007', setMinMax_='0.0-360 step22.5', setValue=22.5, setToolTip='Set the allowed rotation increment contraint.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Stack Similar: ', setObjectName='s011', setMinMax_='0-2 step1', setValue=0, setToolTip='Find Similar shells. <br>state 1: Find similar shells, and pack one of each, ommiting the rest.<br>state 2: Find similar shells, and stack during packing.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='Tolerance: ', setObjectName='s006', setMinMax_='0.0-10 step.1', setValue=1.0, setToolTip='Stack Similar: Stack shells with uv\'s within the given range.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='UDIM: ', setObjectName='s004', setMinMax_='1001-1200 step1', setValue=1001, setToolTip='Set the desired UDIM tile space.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Map Size: ', setObjectName='s005', setMinMax_='512-8192 step512', setValue=2048, setToolTip='UV map resolution.')

		tb007 = self.sb.uv.tb007
		# tb007.ctxMenu.b099.released.connect(lambda: tb007.ctxMenu.s003.setValue(float(pm.mel.texGetTexelDensity(tb007.ctxMenu.s002.value())))) #get and set texel density value.


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.uv.draggable_header.ctxMenu.cmb000

		if index>0: #hide uitk then perform operation
			text = cmb.items[index]
			self.sb.parent().hide()
			if text=='UV Editor':
				mel.eval('TextureViewWindow;') 
			elif text=='UV Set Editor':
				mel.eval('uvSetEditor;')
			elif text=='UV Tool Kit':
				mel.eval('toggleUVToolkit;')
			elif text=='UV Linking: Texture-Centric':
				mel.eval('textureCentricUvLinkingEditor;')
			elif text=='UV Linking: UV-Centric':
				mel.eval('uvCentricUvLinkingEditor;')
			elif text=='UV Linking: Paint Effects/UV':
				mel.eval('pfxUVLinkingEditor;')
			elif text=='UV Linking: Hair/UV':
				mel.eval('hairUVLinkingEditor;')
			elif text=='Flip UV':
				mel.eval("performPolyForceUV flip 1;")
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Display
		'''
		cmb = self.sb.uv.cmb001


	def cmb002(self, index=-1):
		'''Transform
		'''
		cmb = self.sb.uv.cmb002

		if index>0:
			text = cmb.items[index]
			self.sb.parent().hide() #hide hotbox then perform operation
			if text=='Flip U':
				pm.polyFlipUV(flipType=0, local=1, usePivot=1, pivotU=0, pivotV=0)
			elif text=='Flip V':
				pm.polyFlipUV(flipType=1, local=1, usePivot=1, pivotU=0, pivotV=0)
			elif text=='Align U Left':
				pm.mel.performAlignUV('minU')
			elif text=='Align U Middle':
				pm.mel.performAlignUV('avgU')
			elif text=='Align U Right':
				pm.mel.performAlignUV('maxU')
			elif text=='Align U Top':
				pm.mel.performAlignUV('maxV')
			elif text=='Align U Middle':
				pm.mel.performAlignUV('avgV')
			elif text=='Align U Bottom':
				pm.mel.performAlignUV('minV')
			elif text=='Linear Align':
				pm.mel.performLinearAlignUV()
			cmb.setCurrentIndex(0)


	def chk014(self):
		'''Display: Checkered Pattern
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk014.isChecked()

		panel = pm.getPanel(scriptType='polyTexturePlacementPanel')
		pm.textureWindow(panel, edit=1, displayCheckered=state)


	def chk015(self):
		'''Display: Borders
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk015.isChecked()

		borderWidth = pm.optionVar(query='displayPolyBorderEdgeSize')[1]
		borders = pm.polyOptions(displayMapBorder=state, sizeBorder=borderWidth)


	def chk016(self):
		'''Display: Distortion
		'''
		cmb = self.sb.uv.cmb001
		state = cmb.menu_.chk016.isChecked()

		panel = pm.getPanel(scriptType='polyTexturePlacementPanel')
		pm.textureWindow(panel, edit=1, displayDistortion=state)


	def tb000(self, state=None):
		'''Pack UV's

		pm.u3dLayout:
			layoutScaleMode (int),
			multiObject (bool),
			mutations (int),
			packBox (float, float, float, float),
			preRotateMode (int),
			preScaleMode (int),
			resolution (int),
			rotateMax (float),
			rotateMin (float),
			rotateStep (float),
			shellSpacing (float),
			tileAssignMode (int),
			tileMargin (float),
			tileU (int),
			tileV (int),
			translate (bool)
		'''
		tb = self.sb.uv.tb000

		scale = tb.ctxMenu.s009.value()
		rotate = tb.ctxMenu.s010.value()
		rotateStep = tb.ctxMenu.s007.value()
		UDIM = tb.ctxMenu.s004.value()
		mapSize = tb.ctxMenu.s005.value()
		similar = tb.ctxMenu.s011.value()
		tolerance = tb.ctxMenu.s006.value()

		U,D,I,M = [int(i) for i in str(UDIM)]

		sel = self.UvShellSelection() #assure the correct selection mask.
		if similar>0:
			dissimilar = pm.polyUVStackSimilarShells(sel, tolerance=tolerance, onlyMatch=True)
			dissimilarUVs = [s.split() for s in dissimilar] if dissimilar else []
			dissimilarFaces = pm.polyListComponentConversion(dissimilarUVs, fromUV=1, toFace=1)
			pm.u3dLayout(dissimilarFaces, resolution=mapSize, preScaleMode=scale, preRotateMode=rotate, rotateStep=rotateStep, shellSpacing=.005, tileMargin=.005, packBox=[M-1, D, I, U]) #layoutScaleMode (int), multiObject (bool), mutations (int), packBox (float, float, float, float), preRotateMode (int), preScaleMode (int), resolution (int), rotateMax (float), rotateMin (float), rotateStep (float), shellSpacing (float), tileAssignMode (int), tileMargin (float), tileU (int), tileV (int), translate (bool)

		elif similar==2:
			pm.select(dissimilarFaces, toggle=1)
			similarFaces = pm.ls(sl=1)
			pm.polyUVStackSimilarShells(similarFaces, dissimilarFaces, tolerance=tolerance)

		else:
			pm.u3dLayout(sel, resolution=mapSize, preScaleMode=scale, preRotateMode=rotate, rotateStep=rotateStep, shellSpacing=.005, tileMargin=.005, packBox=[M-1, D, I, U]) #layoutScaleMode (int), multiObject (bool), mutations (int), packBox (float, float, float, float), preRotateMode (int), preScaleMode (int), resolution (int), rotateMax (float), rotateMin (float), rotateStep (float), shellSpacing (float), tileAssignMode (int), tileMargin (float), tileU (int), tileV (int), translate (bool)


	@Slots_blender.attr
	def tb001(self, state=None):
		'''Auto Unwrap
		'''
		tb = self.sb.uv.tb001

		standardUnwrap = tb.ctxMenu.chk000.isChecked()
		scaleMode = tb.ctxMenu.chk001.isChecked()
		seamOnly = tb.ctxMenu.chk002.isChecked()
		planarUnwrap = tb.ctxMenu.chk003.isChecked()
		cylindricalUnwrap = tb.ctxMenu.chk004.isChecked()
		sphericalUnwrap = tb.ctxMenu.chk005.isChecked()
		normalBasedUnwrap = tb.ctxMenu.chk006.isChecked()

		selection = pm.ls(selection=1, flatten=1)
		for obj in selection:
			try:
				if seamOnly:
					autoSeam = pm.u3dAutoSeam(obj, s=0, p=1)
					return autoSeam if len(selection)==1 else autoSeam

				elif any((cylindricalUnwrap, sphericalUnwrap, planarUnwrap)):
					unwrapType = 'Planar'
					if cylindricalUnwrap:
						unwrapType = 'Cylindrical'
					elif sphericalUnwrap:
						unwrapType = 'Spherical'
					objFaces = mtk.Cmpt.getComponents('f')
					if not objFaces:
						objFaces = mtk.Cmpt.getComponents(obj, 'f')
					pm.polyProjection(objFaces, type=unwrapType, insertBeforeDeformers=1, smartFit=1)

				elif normalBasedUnwrap:
					pm.mel.texNormalProjection(1, 1, obj) #Normal-Based unwrap

				elif standardUnwrap:
					polyAutoProjection = pm.polyAutoProjection (obj, layoutMethod=0, optimize=1, insertBeforeDeformers=1, scaleMode=scaleMode, createNewMap=False, #Create a new UV set, as opposed to editing the current one, or the one given by the -uvSetName flag.
						projectBothDirections=0, #If "on" : projections are mirrored on directly opposite faces. If "off" : projections are not mirrored on opposite faces. 
						layout=2, #0 UV pieces are set to no layout. 1 UV pieces are aligned along the U axis. 2 UV pieces are moved in a square shape.
						planes=6, #intermediate projections used. Valid numbers are 4, 5, 6, 8, and 12
						percentageSpace=0.2, #percentage of the texture area which is added around each UV piece.
						worldSpace=0) #1=world reference. 0=object reference.

					return polyAutoProjection if len(selection)==1 else polyAutoProjection

			except Exception as error:
				print(error)


	def tb002(self, state=None):
		'''Stack
		'''
		tb = self.sb.uv.tb002

		orient = tb.ctxMenu.chk021.isChecked()
		stackSimilar = tb.ctxMenu.chk022.isChecked()
		tolerance = tb.ctxMenu.s000.value()
		sel = self.UvShellSelection() #assure the correct selection mask.

		if stackSimilar:
			pm.polyUVStackSimilarShells(sel, tolerance=tolerance)
		else:
			pm.mel.texStackShells([])
		if orient:
			pm.mel.texOrientShells()


	def tb003(self, state=None):
		'''Select By Type
		'''
		tb = self.sb.uv.tb003

		back_facing = tb.ctxMenu.chk008.isChecked()
		front_facing = tb.ctxMenu.chk009.isChecked()
		overlapping = tb.ctxMenu.chk010.isChecked()
		nonOverlapping = tb.ctxMenu.chk011.isChecked()
		textureBorders = tb.ctxMenu.chk012.isChecked()
		unmapped = tb.ctxMenu.chk013.isChecked()

		if back_facing:
			pm.mel.selectUVFaceOrientationComponents({}, 0, 2, 1)
		elif front_facing:
			pm.mel.selectUVFaceOrientationComponents({}, 0, 1, 1)
		elif overlapping:
			pm.mel.selectUVOverlappingComponents(1, 0)
		elif nonOverlapping:
			pm.mel.selectUVOverlappingComponents(0, 0)
		elif textureBorders:
			pm.mel.selectUVBorderComponents({}, "", 1)
		elif unmapped:
			pm.mel.selectUnmappedFaces()


	def tb004(self, state=None):
		'''Unfold

		Synopsis: u3dUnfold [flags] [String...]
		Flags:
		  -bi -borderintersection  on|off
		 -ite -iterations          Int
		  -ms -mapsize             Int
		   -p -pack                on|off
		  -rs -roomspace           Int
		  -tf -triangleflip        on|off

		Synopsis: u3dOptimize [flags] [String...]
		Flags:
		  -bi -borderintersection  on|off
		 -ite -iterations          Int
		  -ms -mapsize             Int
		 -pow -power               Int
		  -rs -roomspace           Int
		  -sa -surfangle           Float
		  -tf -triangleflip        on|off
		'''
		tb = self.sb.uv.tb004

		optimize = tb.ctxMenu.chk017.isChecked()
		amount = 1#tb.ctxMenu.s008.value()

		pm.u3dUnfold(iterations=1, pack=0, borderintersection=1, triangleflip=1, mapsize=2048, roomspace=0) #pm.mel.performUnfold(0)

		if optimize:
			pm.u3dOptimize(iterations=amount, power=1, surfangle=1, borderintersection=0, triangleflip=1, mapsize=2048, roomspace=0) #pm.mel.performPolyOptimizeUV(0)


	def tb005(self, state=None):
		'''Straighten Uv
		'''
		tb = self.sb.uv.tb005

		u = tb.ctxMenu.chk018.isChecked()
		v = tb.ctxMenu.chk019.isChecked()
		angle = tb.ctxMenu.s001.value()
		straightenShell = tb.ctxMenu.chk020.isChecked()

		if u and v:
			pm.mel.texStraightenUVs('UV', angle)
		elif u:
			pm.mel.texStraightenUVs('U', angle)
		elif v:
			pm.mel.texStraightenUVs('V', angle)

		if straightenShell:
			pm.mel.texStraightenShell()


	def tb006(self, state=None):
		'''Distribute
		'''
		tb = self.sb.uv.tb006

		u = tb.ctxMenu.chk023.isChecked()
		v = tb.ctxMenu.chk024.isChecked()
		
		if u:
			pm.mel.texDistributeShells(0, 0, "right", []) #'left', 'right'
		if v:
			pm.mel.texDistributeShells(0, 0, "down", []) #'up', 'down'


	def tb007(self, state=None):
		'''Set Texel Density
		'''
		tb = self.sb.uv.tb007

		mapSize = tb.ctxMenu.s002.value()
		density = tb.ctxMenu.s003.value()

		pm.mel.texSetTexelDensity(density, mapSize)


	def b001(self):
		'''Create UV Snapshot
		'''
		pm.mel.UVCreateSnapshot()


	@Slots_blender.undoChunk
	def b002(self):
		'''Transfer UV's
		'''
		selection = pm.ls(orderedSelection=1, flatten=1)
		if len(selection)<2:
			return 'Error: <b>Nothing selected.</b><br>The operation requires the selection of two polygon objects.'

		# pm.undoInfo(openChunk=1)
		set1 = pm.listRelatives(selection[0], children=1)
		set2 = pm.listRelatives(selection[1:], children=1)

		for frm in set1:
			for to in set2:
				if pm.polyEvaluate(frm)==pm.polyEvaluate(to):
					pm.transferAttributes(frm, to, transferUVs=2, sampleSpace=4) #-transferNormals 0 -transferUVs 2 -transferColors 2 -sourceUvSpace "map1" -targetUvSpace "map1" -searchMethod 3-flipUVs 0 -colorBorders 1 ;
					pm.bakePartialHistory(to, prePostDeformers=1)
					set2.remove(to) #remove the obj from the transfer list when an exact match is found.
				elif pm.polyEvaluate(frm, face=1)==pm.polyEvaluate(to, face=1) and pm.polyEvaluate(frm, boundingBox=1)==pm.polyEvaluate(to, boundingBox=1):
					pm.transferAttributes(frm, to, transferUVs=2, sampleSpace=4) #transfer to the object if it is similar, but keep in transfer list in case an exact match is found later.
					pm.bakePartialHistory(to, prePostDeformers=1)

		for remaining in set2:
			print('Error: No match found for: {}.'.format(remaining.name()))
		# pm.undoInfo(closeChunk=1)


	def b005(self):
		'''Cut UV's
		'''
		objects = pm.ls(selection=1, objectsOnly=1, shapes=1, flatten=1)

		for obj in objects:
			sel = pm.ls(obj, sl=1)
			pm.polyMapCut(sel)


	def b011(self):
		'''Sew UV's
		'''
		objects = pm.ls(selection=1, objectsOnly=1, shapes=1, flatten=1)

		for obj in objects:
			sel = pm.ls(obj, sl=1)

			pm.polyMapSew(sel) if len(objects)==1 else pm.polyMapSew(sel)
 

	def moveSelectedToUvSpace(self, u, v, relative=True):
		'''Move sny selected objects to the given u and v coordinates.

		:Parameters:
			u (int): u coordinate.
			v (int): v coordinate.
			relative (bool): Move relative or absolute.
		'''
		sel = self.UvShellSelection() #assure the correct selection mask.

		pm.polyEditUV(sel, u=u, v=v, relative=relative)


	def UvShellSelection(self):
		'''Select all faces of any selected geometry, and switch the component mode to uv shell,
		if the current selection is not maskFacet, maskUv, or maskUvShell.

		:Return:
			(list) the selected faces.
		'''
		selection = pm.ls(sl=1)
		if not selection:
			return 'Error: <b>Nothing selected.<b><br>The operation requires at lease one selected object.'

		objects = pm.ls(selection, objectsOnly=1)
		objectMode = pm.selectMode(query=1, object=1)

		maskFacet = pm.selectType(query=1, facet=1)
		maskUv = pm.selectType(query=1, polymeshUV=1)
		maskUvShell = pm.selectType(query=1, meshUVShell=1)

		if all((objects, objectMode)) or not any((objectMode, maskFacet, maskUv, maskUvShell)):

			for obj in objects:
				pm.selectMode(component=1)
				pm.selectType(meshUVShell=1)
				selection = mtk.Cmpt.getComponents(obj, 'f', flatten=False)
				pm.select(selection, add=True)

		return selection


	def getUvShellSets(self, objects=None, returnType='shells'):
		'''Get All UV shells and their corresponding sets of faces.

		:Parameters:
			objects (obj)(list): Polygon object(s) or Polygon face(s).
			returnType (str): The desired returned type. valid values are: 'shells', 'shellIDs'. If None is given, the full dict will be returned.

		:Return:
			(list)(dict) dependant on the given returnType arg. ex. {0L:[[MeshFace(u'pShape.f[0]'), MeshFace(u'pShape.f[1]')], 1L:[[MeshFace(u'pShape.f[2]'), MeshFace(u'pShape.f[3]')]}
		'''
		if not objects:
			objects = pm.ls(selection=1, objectsOnly=1, transforms=1, flatten=1)

		if not isinstance(objects, (list, set, tuple)):
			objects=[objects]

		objectType = self.getType(objects[0])
		if objectType=='Polygon Face':
			faces = objects
		else:
			faces = mtk.Cmpt.getComponents(objects, 'faces')

		shells={}
		for face in faces:
			shell_Id = pm.polyEvaluate(face, uvShellIds=True)

			try:
				shells[shell_Id[0]].append(face)
			except KeyError:
				try:
					shells[shell_Id[0]]=[face]
				except IndexError:
					pass

		if returnType=='shells':
			shells = list(shells.values())
		elif returnType=='shellIDs':
			shells = shells.keys()

		return shells


	def getUvShellBorderEdges(self, objects):
		'''Get the edges that make up any UV islands of the given objects.

		:Parameters:
			objects (str)(obj)(list): Polygon mesh objects.

		:Return:
			(list) uv border edges.
		'''
		mesh_edges=[]
		for obj in pm.ls(objects, objectsOnly=1):
			try: # Try to get edges from provided objects.
				mesh_edges.extend(pm.ls(pm.polyListComponentConversion(obj, te=True), fl=True, l=True))
			except Exception as error:
				pass

		if len(mesh_edges)<=0: # Error if no valid objects were found
			raise RuntimeError('No valid mesh objects or components were provided.')

		pm.progressWindow(t='Find UV Border Edges', pr=0, max=len(mesh_edges), ii=True) # Start progressWindow
		
		uv_border_edges = list() # Find and return uv border edges
		for edge in mesh_edges:  # Filter through the mesh(s) edges.

			if pm.progressWindow(q=True, ic=True): # Kill if progress window is cancelled
				pm.progressWindow(ep=True)  # End progressWindow
				raise RuntimeError('Cancelled by user.')

			pm.progressWindow(e=True, s=1, st=edge) # Update the progress window status
			
			edge_uvs = pm.ls(pm.polyListComponentConversion(edge, tuv=True), fl=True)
			edge_faces = pm.ls(pm.polyListComponentConversion(edge, tf=True), fl=True)
			if len(edge_uvs) > 2:  # If an edge has more than two uvs, it is a uv border edge.
				uv_border_edges.append(edge)
			elif len(edge_faces) < 2:  # If an edge has less than 2 faces, it is a border edge.
				uv_border_edges.append(edge)

		pm.progressWindow(ep=True) # End progressWindow

		return uv_border_edges









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Scene(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		dh = self.sb.scene.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Scene Editors')

		t000 = self.sb.scene.t000
		t000.ctxMenu.add('QCheckBox', setText='Ignore Case', setObjectName='chk000', setToolTip='Search case insensitive.')
		t000.ctxMenu.add('QCheckBox', setText='Regular Expression', setObjectName='chk001', setToolTip='When checked, regular expression syntax is used instead of the default \'*\' and \'|\' wildcards.')

		tb000 = self.sb.scene.tb000
		tb000.ctxMenu.add('QComboBox', addItems=['capitalize', 'upper', 'lower', 'swapcase', 'title'], setObjectName='cmb001', setToolTip='Set desired python case operator.')

		tb001 = self.sb.scene.tb001
		tb001.ctxMenu.add('QCheckBox', setText='Alphanumeric', setObjectName='chk005', setToolTip='When True use an alphanumeric character as a suffix when there is less than 26 objects else use integers.')
		tb001.ctxMenu.add('QCheckBox', setText='Strip Trailing Integers', setObjectName='chk002', setChecked=True, setToolTip="Strip any trailing integers. ie. '123' of 'cube123'")
		tb001.ctxMenu.add('QCheckBox', setText='Strip Trailing Alphanumeric', setObjectName='chk003', setChecked=True, setToolTip="Strip any trailing uppercase alphanumeric chars that are prefixed with an underscore.  ie. 'A' of 'cube_A'")
		tb001.ctxMenu.add('QCheckBox', setText='Reverse', setObjectName='chk004', setToolTip='Reverse the naming order. (Farthest object first)')


	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.scene.draggable_header


	def t000(self, state=None):
		'''Find
		'''
		t000 = self.sb.scene.t000









# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


#deprecated:


# !/usr/bin/python
# coding=utf-8
import sys, os

from PySide2 import QtCore, QtGui, QtWidgets

from pythontk.Iter import makeList
from uitk.switchboard import Switchboard
from uitk.overlay import Overlay
from uitk.events import EventFactoryFilter, MouseTracking
from uitk.widgets import rwidgets


class Tcl(QtWidgets.QStackedWidget):
	'''Tcl is a marking menu based on a QStackedWidget.
	Gets and sets signal connections (through the switchboard module).
	Initializes events for child widgets using the eventFilter module.
	Plots points for paint events in the overlay module.

	The various ui's are set by calling 'setUi' with the intended ui name string. ex. Tcl().setUi('polygons')
	'''
	_mousePressPos = QtCore.QPoint()
	_key_show_release = QtCore.Signal()

	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.

	def __init__(self, parent=None, key_show='Key_F12', preventHide=False, slotLoc=''):
		'''
		:Parameters:
			parent (obj): The parent application's top level window instance. ie. the Maya main window.
		'''
		super().__init__(parent)

		self.sb = Switchboard(self, uiLoc='ui', widgetLoc=rwidgets, slotLoc=slotLoc, preloadUi=True)

		self.key_show = getattr(QtCore.Qt, key_show)
		self.key_undo = QtCore.Qt.Key_Z
		self.key_close = QtCore.Qt.Key_Escape
		self.preventHide = preventHide

		# self.app.setDoubleClickInterval(400)
		# self.app.setKeyboardInputInterval(400)
		# self.app.focusChanged.connect(self.focusChanged)

		self.setWindowFlags(QtCore.Qt.Tool|QtCore.Qt.FramelessWindowHint) #|QtCore.Qt.WindowStaysOnTopHint
		self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
		self.setAttribute(QtCore.Qt.WA_SetStyle) #Indicates that the widget has a style of its own.

		self.overlay = Overlay(self, antialiasing=True) #Paint events are handled by the overlay module.
		self.eventFilter = EventFactoryFilter(self, eventNamePrefix='ef_', forwardEventsTo=self)
		self.mouseTracking = MouseTracking(self)


	def initUi(self, ui):
		'''Initialize the given ui.

		:Parameters:
			ui (obj): The ui to initialize.
		'''
		self.initWidgets(ui)

		if ui.level<3: #stacked ui.
			ui.setParent(self)
			self.addWidget(ui) #add the ui to the stackedLayout.

		else: #popup ui.
			ui.setParent(self.parent())
			ui.setWindowFlags(QtCore.Qt.Tool|QtCore.Qt.FramelessWindowHint)
			ui.setAttribute(QtCore.Qt.WA_TranslucentBackground)

			self._key_show_release.connect(ui.hide)

		ui.isInitialized = True


	def setUi(self, ui):
		'''Set the stacked Widget's index to the given ui.

		:Parameters:
			ui (str)(obj): The ui or name of the ui to set the stacked widget index to.
		'''
		assert isinstance(ui, (str, QtWidgets.QWidget)), f'# Error: {__file__} in setUi\n#\tIncorrect datatype: {type(ui).__name__}'

		ui = self.sb.getUi(ui) #Get the ui of the given name, and set it as the current ui in the switchboard module.
		ui.connected = True

		if not ui.isInitialized:
			self.initUi(ui)

		if ui.level<3: #stacked ui top level window.
			self.setCurrentWidget(ui) #set the stacked widget to the given ui.
			self.resize(ui.sizeX, ui.sizeY) #The ui sizes for individual ui's are stored in sizeX and sizeY properties. Otherwise size would be constrained to the largest widget in the stack)

		else: #popup ui.
			ui.show()
			ui.resize(ui.minimumSizeHint()) #ui.adjustSize()
			self.sb.moveAndCenterWidget(ui, QtGui.QCursor.pos(), offsetY=4) #move to cursor position and offset slightly.
			ui.activateWindow() #activate the popup ui before hiding the stacked layout.
			self.hide()


	def setSubUi(self, ui, w):
		'''Set the stacked widget's index to the submenu associated with the given widget.
		Positions the new ui to line up with the previous ui's button that called the new ui.

		:Parameters:
			ui (obj): The submenu ui to set as current.
			w (obj): The widget that called this method.
		'''
		if not ui or ui==self.sb.currentUi:
			return

		self.overlay.addToDrawPath(w)
		p1 = w.mapToGlobal(w.rect().center()) #the widget position before submenu change.

		self.setUi(ui) #switch the stacked widget to the given submenu.

		w2 = getattr(self.currentWidget(), w.name) #get the widget of the same name in the new ui.
		p2 = w2.mapToGlobal(w2.rect().center()) #widget position after submenu change.
		currentPos = self.mapToGlobal(self.pos())
		self.move(self.mapFromGlobal(currentPos +(p1 - p2))) #currentPos + difference

		if ui not in self.sb.getPrevUi(asList=True): #if the submenu ui called for the first time:
			self.cloneWidgetsAlongPath(ui) #re-construct any widgets from the previous ui that fall along the plotted path.

		self.overlay.removeFromPath(ui) #remove entrys from widget and draw paths when moving back down levels in the ui.
		# self.resize(ui.sizeX, ui.sizeY)


	def returnToStart(self):
		'''Return the stacked widget to it's starting index.
		'''
		prevUi = self.sb.getPrevUi(omitLevel=2)
		self.setUi(prevUi) #return the stacked widget to it's previous ui.

		self.move(self.overlay.drawPathStartPos - self.rect().center())

		self.overlay.clearDrawPath()


	def cloneWidgetsAlongPath(self, ui):
		'''Re-constructs the relevant buttons from the previous ui for the new ui, and positions them.
		Initializes the new buttons by adding them to the switchboard.
		The previous widget information is derived from the widget and draw paths.

		:Parameters:
			ui (obj): The ui in which to copy the widgets to.
		'''
		for prevWgt, prevPos, drawPos in self.overlay.drawPath:
			state = True if prevWgt.objectName()!='return_area' else False
			w = self.sb.PushButton(ui, copy_=prevWgt, setPosition_=prevPos, setVisible=state)
			self.initWidgets(ui, w) #initialize the widget to set things like the event filter and styleSheet.
			self.sb.connectSlots(ui, w)



	# ---------------------------------------------------------------------------------------------
	# 	Stacked Widget Event handling:
	# ---------------------------------------------------------------------------------------------
	def sendKeyPressEvent(self, key, modifier=QtCore.Qt.NoModifier):
		'''
		'''
		self.grabKeyboard()
		event = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, modifier)
		self.keyPressEvent(event) # self.app.postEvent(self, event)


	def keyPressEvent(self, event):
		'''A widget must call setFocusPolicy() to accept focus initially, and have focus, in order to receive a key press event.
		'''
		if event.isAutoRepeat():
			return

		# modifiers = self.app.keyboardModifiers()

		# if event.key()==self.key_show:
		# 	self.show()

		elif event.key()==self.key_close:
			self.close()

		super().keyPressEvent(event)


	def keyReleaseEvent(self, event):
		'''A widget must accept focus initially, and have focus, in order to receive a key release event.
		'''
		if event.isAutoRepeat():
			return

		modifiers = self.app.keyboardModifiers()

		if event.key()==self.key_show and not modifiers==QtCore.Qt.ControlModifier:
			self._key_show_release.emit()
			self.releaseKeyboard()
			self.hide()

		super().keyReleaseEvent(event)


	def mousePressEvent(self, event):
		'''
		'''
		modifiers = self.app.keyboardModifiers()

		if self.sb.currentUi.level<3:
			self.move(self.sb.getCenter(self))

			if not modifiers:
				if event.button()==QtCore.Qt.LeftButton:
					self.setUi('cameras')

				elif event.button()==QtCore.Qt.MiddleButton:
					self.setUi('editors')

				elif event.button()==QtCore.Qt.RightButton:
					self.setUi('main')

		super().mousePressEvent(event)


	def mouseMoveEvent(self, event):
		'''If mouse tracking is switched off, mouse move events only occur if 
		a mouse button is pressed while the mouse is being moved. If mouse tracking 
		is switched on, mouse move events occur even if no mouse button is pressed.
		'''
		self.mouseTracking.track(self.sb.currentUi.widgets)

		super().mouseMoveEvent(event)


	def mouseReleaseEvent(self, event):
		'''
		'''
		self.setUi('init')

		super().mouseReleaseEvent(event)


	def mouseDoubleClickEvent(self, event):
		'''The widget will also receive mouse press and mouse release events 
		in addition to the double click event. If another widget that overlaps 
		this widget disappears in response to press or release events, 
		then this widget will only receive the double click event.
		'''
		modifiers = self.app.keyboardModifiers()

		if self.sb.currentUi.level<3:
			if event.button()==QtCore.Qt.LeftButton:

				if modifiers in (QtCore.Qt.ControlModifier, QtCore.Qt.ControlModifier | QtCore.Qt.AltModifier):
					self.repeatLastCommand()
				else:
					self.repeatLastCameraView()

			elif event.button()==QtCore.Qt.MiddleButton:
				pass

			elif event.button()==QtCore.Qt.RightButton:
				self.repeatLastUi()

		super().mouseDoubleClickEvent(event)


	def focusChanged(self, old, new):
		'''Called on focus events.

		:Parameters:
			old (obj): The widget with previous focus.
			new (obj): The widget with current focus.
		'''
		try:
			new.grabKeyboard()
		except AttributeError as error:
			self.setFocus()

		if not self.isActiveWindow():
			self.hide()


	def show(self, ui='init', profile=False):
		'''Sets the widget as visible.

		:Parameters:
			ui (str)(obj): Show the given ui.
			profile (bool): Prints the total running time, times each function separately, 
				and tells you how many times each function was called.
		'''
		self.sendKeyPressEvent(self.key_show)

		if profile:
			import cProfile
			cProfile.run(f'self.setUi({self.sb.getUi(ui).name})')
		else:
			self.setUi(ui)

		# self.activateWindow()
		super().show()


	def showEvent(self, event):
		'''Non-spontaneous show events are sent to widgets immediately before they are shown.
		'''
		if self.sb.currentUi.level==0:
			self.move(self.sb.getCenter(self))

		super().showEvent(event)


	def hide(self, force=False):
		'''Sets the widget as invisible.
		Prevents hide event under certain circumstances.

		:Parameters:
			force (bool): override preventHide.
		'''
		if force or not self.preventHide:
			# print ('mouseGrabber:', self.mouseGrabber()) #Returns the widget that is currently grabbing the mouse input. else: None 
			super().hide()


	def hideEvent(self, event):
		'''Hide events are sent to widgets immediately after they have been hidden.
		'''
		try:
			self.mouseGrabber().releaseMouse()
		except AttributeError as error: #'NoneType' object has no attribute 'releaseMouse'
			pass

		super().hideEvent(event)



	# ---------------------------------------------------------------------------------------------
	# child widget event handling:
	# ---------------------------------------------------------------------------------------------
	ef_widgetTypes = [ #install an event filter for the given widget types.
		'QMainWindow',
		'QWidget', 
		'QAction', 
		'QLabel', 
		'QPushButton', 
		'QListWidget', 
		'QTreeWidget', 
		'QComboBox', 
		'QSpinBox',
		'QDoubleSpinBox',
		'QCheckBox',
		'QRadioButton',
		'QLineEdit',
		'QTextEdit',
		'QProgressBar',
		'QMenu',
	]
	def initWidgets(self, ui, widgets=None):
		'''Set Initial widget states.

		:Parameters:
			ui (obj): The ui to init widgets of.
			widgets (str)(list): <QWidgets> if no arg is given, the operation will be performed on all widgets of the given ui name.
		'''
		if widgets is None:
			widgets = ui.widgets #get all widgets for the given ui.

		for w in makeList(widgets): #if 'widgets' isn't a list, convert it to one.

			if w not in ui.widgets:
				ui.addWidgets(w)

			#set styleSheet
			if ui.level>2 or ui.isSubmenu and not w.prefix=='i': #if submenu and objectName doesn't start with 'i':
				self.sb.setStyle(w, style='dark', backgroundOpacity=0)
			else:
				self.sb.setStyle(w, backgroundOpacity=0)

			if w.derivedType in self.ef_widgetTypes:
				# print (widgetName if widgetName else widget)
				if ui.level<3 or w.type=='QMainWindow':
					w.installEventFilter(self.eventFilter)

				if w.derivedType in ('QPushButton', 'QLabel'): #widget types to resize and center.
					if ui.level<=2:
						self.sb.resizeAndCenterWidget(w)

				elif w.derivedType=='QWidget': #widget types to set an initial state as hidden.
					if w.prefix=='w' and ui.level==1: #prefix returns True if widgetName startswith the given prefix, and is followed by three integers.
						w.setVisible(False)


	def ef_showEvent(self, w, event):
		'''
		'''
		if w.name=='info':
			self.sb.resizeAndCenterWidget(w)

		if w.type in ('ComboBox', 'ListWidget'):
			try: #call the class method associated with the current widget.
				w.method()
			except (AttributeError, TypeError) as error:
				print (f'# Error: {__file__} in ef_showEvent\n#\t{error}.')
				pass

		w.__class__.showEvent(w, event)


	def ef_hideEvent(self, w, event):
		'''
		'''

		w.__class__.hideEvent(w, event)


	def ef_enterEvent(self, w, event):
		'''
		'''
		if w.type=='QWidget':
			if w.prefix=='w':
				w.setVisible(True) #set visibility

		elif w.derivedType=='QPushButton':
			if w.prefix=='i': #set the stacked widget.
				subUi = self.sb.getUi(w.whatsThis(), level=2)
				self.setSubUi(subUi, w)

			elif w.name=='return_area':
				self.returnToStart()

		if w.prefix=='chk':
			if w.ui.isSubmenu:
				w.click()

		w.__class__.enterEvent(w, event)


	def ef_leaveEvent(self, w, event):
		'''
		'''
		if w.type=='QWidget':
			if w.prefix=='w':
				w.setVisible(False) #set visibility

		w.__class__.leaveEvent(w, event)


	def ef_mousePressEvent(self, w, event):
		'''
		'''
		self._mousePressPos = event.globalPos() #mouse positon at press
		self.__mouseMovePos = event.globalPos() #mouse move position from last press

		w.__class__.mousePressEvent(w, event)


	def ef_mouseMoveEvent(self, w, event):
		'''
		'''
		try:# if hasattr(self, '__mouseMovePos'):
			globalPos = event.globalPos()
			diff = globalPos - self.__mouseMovePos
			self.__mouseMovePos = globalPos
		except AttributeError as error:
			pass

		w.__class__.mouseMoveEvent(w, event)


	def ef_mouseReleaseEvent(self, w, event):
		'''
		'''
		if w.underMouse(): #if self.widget.rect().contains(event.pos()): #if mouse over widget:
			if w.derivedType=='QPushButton':
				if w.prefix=='i': #ie. 'i012'
					#hide/show ui groupboxes according to the buttons whatsThis formatting.
					name, *gbNames = w.whatsThis().split('#') #get any groupbox names that were prefixed by '#'.
					groupBoxes = self.sb.getWidgetsByType('QGroupBox', name)
					[gb.hide() if gbNames and not gb.name in gbNames else gb.show() for gb in groupBoxes] #show only groupboxes with those names if any were given, else show all.
					# self.app.processEvents() #the minimum size is not computed until some events are processed in the event loop.
					self.setUi(name)

				elif w.prefix=='v':
					if w.ui.name=='cameras':
						self.prevCamera(add=w.method)
					#send click signal on mouseRelease.
					w.click()

				elif w.prefix in ('b','tb'):
					if w.ui.isSubmenu:
						w.click()
						self.hide()

		w.__class__.mouseReleaseEvent(w, event)


	def ef_sendKeyPressEvent(self, w, key, modifier=QtCore.Qt.NoModifier):
		'''
		'''
		w.grabKeyboard()
		event = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, modifier)
		self.ef_keyPressEvent(w, event)


	def ef_keyPressEvent(self, w, event):
		'''A widget must call setFocusPolicy() to accept focus initially, and have focus, in order to receive a key press event.
		'''
		if not event.isAutoRepeat():
			# print (f'ef_keyPressEvent: {w}: {event}')
			modifiers = self.app.keyboardModifiers()

			if event.key()==self.key_close:
				self.close()

		w.__class__.keyPressEvent(w, event)


	def ef_keyReleaseEvent(self, w, event):
		'''A widget must accept focus initially, and have focus, in order to receive a key release event.
		'''
		if not event.isAutoRepeat():
			# print (f'ef_keyReleaseEvent: {e}: {event}')
			modifiers = self.app.keyboardModifiers()

			if event.key()==self.key_show and not modifiers==QtCore.Qt.ControlModifier:
				if w.name=='mainWindow':#w.type=='QMainWindow':
					if w.ui.level>2:
						self._key_show_release.emit()
						w.releaseKeyboard()

		w.__class__.keyReleaseEvent(w, event)


	# ---------------------------------------------------------------------------------------------
	# 	
	# ---------------------------------------------------------------------------------------------
	def repeatLastCommand(self):
		'''Repeat the last stored command.
		'''
		method = self.sb.prevCommand

		if callable(method):
			method()
		else:
			print('# Result: No recent commands in history. #')


	def repeatLastCameraView(self):
		'''Show the previous camera view.
		'''
		method = self.prevCamera()
		if method:
			method()
			method_ = self.prevCamera(allowCurrent=True, asList=1)[-2][0]
		else:
			method_ = self.sb.cameras.slots.v004 #get the persp camera.
			method_()
		self.prevCamera(add=method_) #store the camera view


	def repeatLastUi(self):
		'''Open the last used level 3 menu.
		'''
		prevUi = self.sb.getPrevUi(omitLevel=[0,1,2])
		if prevUi:
			self.setUi(prevUi)
		else:
			print('# Result: No recent menus in history. #')


	_cameraHistory = []
	def prevCamera(self, docString=False, method=False, allowCurrent=False, asList=False, add=None):
		'''
		:Parameters:
			docString (bool): return the docString of last camera command. Default is off.
			method (bool): return the method of last camera command. Default is off.
			allowCurrent (bool): allow the current camera. Default is off.
			add (str)(obj): Add a method, or name of method to be used as the command to the current camera.  (if this flag is given, all other flags are invalidated)

		:Return:
			if docString: 'string' description (derived from the last used camera command's docString) (asList: [string list] all docStrings, in order of use)
			if method: method of last used camera command. (asList: [<method object> list} all methods, in order of use)
			if asList: list of lists with <method object> as first element and <docString> as second. ie. [[<v001>, 'camera: persp']]
			else : <method object> of the last used command
		'''
		if add: #set the given method as the current camera.
			if not callable(add):
				add = self.sb.getMethod('cameras', add)
			docString = add.__doc__
			prevCameraList = self.prevCamera(allowCurrent=True, asList=1)
			if not prevCameraList or not [add, docString]==prevCameraList[-1]: #ie. do not append perp cam if the prev cam was perp.
				prevCameraList.append([add, docString]) #store the camera view
			return

		self._cameraHistory = self._cameraHistory[-20:] #keep original list length restricted to last 20 elements

		hist = self._cameraHistory
		[hist.remove(l) for l in hist[:] if hist.count(l)>1] #remove any previous duplicates if they exist; keeping the last added element.

		if not allowCurrent:
			hist = hist[:-1] #remove the last index. (currentName)

		if asList:
			if docString and not method:
				try:
					return [i[1] for i in hist]
				except:
					return None
			elif method and not docString:
				try:
					return [i[0] for i in hist]
				except:
					return ['# No commands in history. #']
			else:
				return hist

		elif docString:
			try:
				return hist[-1][1]
			except:
				return ''

		else:
			try:
				return hist[-1][0]
			except:
				return None

# --------------------------------------------------------------------------------------------









# --------------------------------------------------------------------------------------------

if __name__ == '__main__':

	main = Tcl()
	main.show('init')

	exit_code = main.app.exec_()
	if exit_code != -1:
		sys.exit(exit_code) # run app, show window, wait for input, then terminate program with a status code returned from app.

#module name
# print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------





# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------

# class Instance():
# 	'''Manage multiple instances of the Tcl ui.
# 	'''
# 	instances={}

# 	def __init__(self, parent=None, preventHide=False, key_show='Key_F12'):
# 		'''
# 		'''
# 		self.Class = Tcl

# 		self.parent = parent
# 		self.preventHide = preventHide
# 		self.key_show = key_show

# 		self.activeWindow_ = None


# 	@property
# 	def instances_(self):
# 		'''Get all instances as a dictionary with the names as keys, and the window objects as values.
# 		'''
# 		return {k:v for k,v in self.instances.items() if not any([v.isVisible(), v==self.activeWindow_])}


# 	def _getInstance(self):
# 		'''Internal use. Returns a new instance if one is running and currently visible.
# 		Removes any old non-visible instances outside of the current 'activeWindow_'.
# 		'''
# 		if self.activeWindow_ is None or self.activeWindow_.isVisible():
# 			name = 'uitk'+str(len(self.instances_))
# 			setattr(self, name, self.Class(self.parent, self.preventHide, self.key_show)) #set the instance as a property using it's name.
# 			self.activeWindow_ = getattr(self, name)
# 			self.instances_[name] = self.activeWindow_

# 		return self.activeWindow_


# 	def show(self, uiName=None, active=True):
# 		'''Sets the widget as visible.

# 		:Parameters:
# 			uiName (str): Show the ui of the given name.
# 			active (bool): Set as the active window.
# 		'''
# 		inst = self._getInstance()
# 		inst.show(uiName=uiName, active=active)






# class Worker(QtCore.QObject):
# 	'''Send and receive signals from the GUI allowing for events to be 
# 	triggered from a separate thread.

# 	ex. self.worker = Worker()
# 		self.thread = QtCore.QThread()
# 		self.worker.moveToThread(self.thread)
# 		self.worker.finished.connect(self.thread.quit)
# 		self.thread.started.connect(self.worker.start)
# 		self.thread.start()
# 		self.worker.stop()
# 	'''
# 	started = QtCore.Signal()
# 	updateProgress = QtCore.Signal(int)
# 	finished = QtCore.Signal()

# 	def __init__(self, parent=None):
# 		super().__init__(parent)

# 	def start(self):
# 		self.started.emit()

# 	def stop(self):
# 		self.finished.emit()


# self.worker = Worker()
# self.thread = QtCore.QThread()
# self.worker.moveToThread(self.thread)

# self.worker.finished.connect(self.thread.quit)
# self.thread.started.connect(self.worker.start)

# # self.loadingIndicator = self.sb.LoadingIndicator(color='white', start=True, setPosition_='cursor')
# self.loadingIndicator = self.sb.GifPlayer(setPosition_='cursor')
# self.worker.started.connect(self.loadingIndicator.start)
# self.worker.finished.connect(self.loadingIndicator.stop)
# self.thread.start()

# import time #threading test
# for _ in range(11):
# 	time.sleep(.25)

#code
# self.worker.stop()


		# if self.addUi(ui, query=True): #if the ui has not yet been added to the widget stack.
		# 	self.addUi(ui) #initialize the parent ui if not done so already.

	# def addUi(self, ui, query=False):
	# 	'''Initializes the ui of the given name and it's dependancies.

	# 	:Parameters:
	# 		ui (obj): The ui widget to be added to the layout stack.
	# 		query (bool): Check whether the ui widget has been added.

	# 	:Return:
	# 		(bool) When queried.
	# 	'''
	# 	if query:
	# 		return self.indexOf(ui)<0

	# 	for i in reversed(range(4)): #in order of uiLevel heirarchy (top-level parent down).
	# 		ui_ = self.sb.getUi(ui, level=i)
	# 		if ui_:
	# 			name = self.sb.getUiName(ui_)

	# 			if self.addUi(ui_, query=True): #if the ui has not yet been added to the widget stack.
	# 				self.addWidget(ui_) #add the ui to the stackedLayout.
	# 				self.initWidgets(name)


# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.dynLayout import DynLayout



class DynLayout_maya(DynLayout, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.dynLayout.draggable_header.ctxMenu.cmb000
		items = []
		cmb.addItems_(items, '')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.dynLayout.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------